# Generated by Django 3.0.8 on 2020-08-05 11:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0057_page_locale_fields_notnull'),
        ('wagtail_localize', '0004_one_source_per_objectlocale'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='translation',
            unique_together={('source', 'target_locale')},
        ),
        migrations.RemoveField(
            model_name='translation',
            name='object',
        ),
    ]



from django.contrib.admin.utils import quote
from django.urls import reverse
from rest_framework import fields, serializers, viewsets

from wagtail.snippets.views.snippets import get_snippet_model_from_url_params


class BaseSnippetSerializer(serializers.ModelSerializer):
    id = fields.ReadOnlyField(source='pk')
    title = fields.SerializerMethodField()
    edit_url = fields.SerializerMethodField()

    def get_title(self, instance):
        return str(instance)

    def get_edit_url(self, instance):
        return reverse('wagtailsnippets:edit', args=[instance._meta.app_label, instance._meta.model_name, quote(instance.id)])


class SnippetViewSet(viewsets.ModelViewSet):
    def get_model(self):
        return get_snippet_model_from_url_params(self.kwargs['app_label'], self.kwargs['model_name'])

    def get_queryset(self):
        return self.get_model()._default_manager.all()

    def get_serializer_class(self):
        model = self.get_model()

        return type(model.__name__ + 'Serializer', (BaseSnippetSerializer, ), {
            'Meta': type('Meta', (object, ), {
                'model': model,
                'fields': ['id', 'title', 'edit_url']
            }),
        })



from django import forms

from django.contrib import messages
from django.contrib.admin.utils import quote
from django.core.exceptions import PermissionDenied, ValidationError
from django.db import transaction
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.utils.translation import gettext as _, gettext_lazy
from django.views.generic import TemplateView
from django.views.generic.detail import SingleObjectMixin
from wagtail.admin.views.pages.utils import get_valid_next_url_from_request
from wagtail.core.models import Page

from wagtail_localize.models import TranslationSource


class UpdateTranslationsForm(forms.Form):
    publish_translations = forms.BooleanField(
        label=gettext_lazy("Publish immediately"),
        help_text=gettext_lazy("This will apply the updates and publish immediately, before any new translations happen."),
        required=False,
    )


class UpdateTranslationsView(SingleObjectMixin, TemplateView):
    template_name = "wagtail_localize/admin/update_translations.html"
    title = gettext_lazy("Update existing translations")

    def get_object(self):
        return get_object_or_404(TranslationSource, id=self.kwargs['translation_source_id'])

    def get_title(self):
        return self.title

    def get_subtitle(self):
        return self.object.object_repr

    def get_form(self):
        if self.request.method == 'POST':
            return UpdateTranslationsForm(self.request.POST)
        else:
            return UpdateTranslationsForm()

    def get_success_message(self):
        return _("Successfully updated translations for '{object}'").format(object=self.object.object_repr)

    def get_success_url(self):
        return get_valid_next_url_from_request(self.request)

    def get_default_success_url(self):
        instance = self.object.get_source_instance()

        if isinstance(instance, Page):
            return reverse("wagtailadmin_explore", args=[instance.get_parent().id])
        else:
            return reverse("wagtailsnippets:edit", args=[instance._meta.app_label, instance._meta.model_name, quote(instance.pk)])

    def get_edit_url(self, instance):
        if isinstance(instance, Page):
            return reverse("wagtailadmin_pages:edit", args=[instance.id])
        else:
            return reverse("wagtailsnippets:edit", args=[instance._meta.app_label, instance._meta.model_name, quote(instance.pk)])

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({
            "translations": [
                {
                    "title": str(translation.get_target_instance()),
                    "locale": translation.target_locale,
                    "edit_url": self.get_edit_url(translation.get_target_instance()),
                }
                for translation in self.object.translations.filter(enabled=True).select_related("target_locale")
            ],
            "last_sync_date": self.object.last_updated_at,
            "form": self.get_form(),
            "next_url": self.get_success_url(),
            "back_url": self.get_success_url() or self.get_default_success_url(),
        })
        return context

    def post(self, request, **kwargs):
        form = self.get_form()

        if form.is_valid():
            with transaction.atomic():
                self.object.update_from_db()

                if form.cleaned_data['publish_translations']:
                    for translation in self.object.translations.filter(enabled=True).select_related("target_locale"):
                        try:
                            translation.save_target(user=request.user, publish=True)
                        except ValidationError:
                            pass

                # TODO: Button that links to page in translations report when we have it
                messages.success(
                    self.request, self.get_success_message()
                )

                return redirect(self.get_success_url() or self.get_default_success_url())

        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)

    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perms(['wagtail_localize.submit_translation']):
            raise PermissionDenied

        self.object = self.get_object()
        return super().dispatch(request, *args, **kwargs)



# Generated by Django 3.1 on 2020-08-28 10:00

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize', '0009_stringtranslation_errors'),
    ]

    operations = [
        migrations.CreateModel(
            name='OverridableSegment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveIntegerField()),
                ('data_json', models.TextField()),
                ('context', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='wagtail_localize.translationcontext')),
                ('source', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wagtail_localize.translationsource')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.contrib import messages
from django.urls import reverse


def assert_permission_denied(self, response):
    # Checks for Wagtail's permission denied response
    self.assertRedirects(response, reverse('wagtailadmin_home'))

    raised_messages = [
        (message.level_tag, message.message)
        for message in messages.get_messages(response.wsgi_request)
    ]
    self.assertIn(('error', 'Sorry, you do not have permission to access this area.\n\n\n\n\n'), raised_messages)



from django.apps import AppConfig


class WagtailLocalizeTestAppConfig(AppConfig):
    label = "wagtail_localize_test"
    name = "wagtail_localize.test"
    verbose_name = "Wagtail Localize Test app"



from django import forms

from django.contrib import messages
from django.contrib.admin.utils import unquote
from django.core.exceptions import PermissionDenied, ValidationError
from django.db import transaction
from django.http import Http404
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.utils.translation import gettext as _, gettext_lazy, ngettext
from django.views.generic import TemplateView
from django.views.generic.detail import SingleObjectMixin
from wagtail.admin.views.pages.utils import get_valid_next_url_from_request
from wagtail.core.models import Page, Locale, TranslatableMixin
from wagtail.snippets.views.snippets import get_snippet_model_from_url_params

from wagtail_localize.models import Translation, TranslationSource


class SubmitTranslationForm(forms.Form):
    # Note: We don't actually use select_all in Python., is is just the
    # easiest way to add the widget to the form. It's controlled in JS.
    select_all = forms.BooleanField(label=gettext_lazy("Select all"), required=False)
    locales = forms.ModelMultipleChoiceField(
        label=gettext_lazy("Locales"), queryset=Locale.objects.none(), widget=forms.CheckboxSelectMultiple
    )
    include_subtree = forms.BooleanField(required=False, help_text=gettext_lazy("All child pages will be created."))

    def __init__(self, instance, *args, **kwargs):
        super().__init__(*args, **kwargs)

        hide_include_subtree = True

        if isinstance(instance, Page):
            descendant_count = instance.get_descendants().count()

            if descendant_count > 0:
                hide_include_subtree = False
                self.fields["include_subtree"].label = ngettext("Include subtree ({} page)", "Include subtree ({} pages)", descendant_count).format(descendant_count)

        if hide_include_subtree:
            self.fields["include_subtree"].widget = forms.HiddenInput()

        existing_translations = instance.get_translations(inclusive=True)

        # Don't count page aliases as existing translations. We can convert aliases into properly translated pages
        if isinstance(instance, Page):
            existing_translations = existing_translations.exclude(alias_of__isnull=False)

        self.fields["locales"].queryset = Locale.objects.exclude(
            id__in=existing_translations.values_list('locale_id', flat=True)
        )

        # Using len() instead of count() here as we're going to evaluate this queryset
        # anyway and it gets cached so it'll only have one query in the end.
        if len(self.fields["locales"].queryset) < 2:
            self.fields["select_all"].widget = forms.HiddenInput()


class TranslationCreator:
    """
    A class that provides a create_translations method.

    Call create_translations for each object you want to translate and this will submit
    that object and any dependencies as well.

    This class will track the objects that have already submitted so an object doesn't
    get submitted twice.
    """
    def __init__(self, user, target_locales):
        self.user = user
        self.target_locales = target_locales
        self.seen_objects = set()

    def create_translations(self, instance, include_related_objects=True):
        if isinstance(instance, Page):
            instance = instance.specific

        if instance.translation_key in self.seen_objects:
            return
        self.seen_objects.add(instance.translation_key)

        source, created = TranslationSource.get_or_create_from_instance(instance)

        # Add related objects
        # Must be before translation records or those translation records won't be able to create
        # the objects because the dependencies haven't been created
        if include_related_objects:
            for related_object_segment in source.relatedobjectsegment_set.all():
                related_instance = related_object_segment.object.get_instance(instance.locale)

                # Limit to one level of related objects, since this could potentially pull in a lot of stuff
                self.create_translations(related_instance, include_related_objects=False)

        # Set up translation records
        for target_locale in self.target_locales:
            # Create translation if it doesn't exist yet, re-enable if translation was disabled
            # Note that the form won't show this locale as an option if the translation existed
            # in this langauge, so this shouldn't overwrite any unmanaged translations.
            translation, created = Translation.objects.update_or_create(
                source=source,
                target_locale=target_locale,
                defaults={
                    'enabled': True
                }
            )

            try:
                translation.save_target(user=self.user)
            except ValidationError:
                pass


class SubmitTranslationView(SingleObjectMixin, TemplateView):
    template_name = "wagtail_localize/admin/submit_translation.html"
    title = gettext_lazy("Translate")

    def get_title(self):
        return self.title

    def get_subtitle(self):
        return str(self.object)

    def get_form(self):
        if self.request.method == 'POST':
            return SubmitTranslationForm(self.object, self.request.POST)
        else:
            initial = None
            if self.request.GET.get('select_locale', None):
                select_locale = Locale.objects.filter(language_code=self.request.GET['select_locale']).first()
                if select_locale:
                    initial = {'locales': [select_locale]}

            return SubmitTranslationForm(self.object, initial=initial)

    def get_success_url(self):
        return get_valid_next_url_from_request(self.request)

    def get_default_success_url(self):
        pass

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({
            "form": self.get_form(),
            "next_url": self.get_success_url(),
            "back_url": self.get_success_url() or self.get_default_success_url(),
        })
        return context

    def post(self, request, **kwargs):
        form = self.get_form()

        if form.is_valid():
            with transaction.atomic():
                translator = TranslationCreator(self.request.user, form.cleaned_data["locales"])
                translator.create_translations(self.object)

                # Now add the sub tree (if the obj is a page)
                if isinstance(self.object, Page):
                    if form.cleaned_data["include_subtree"]:
                        def _walk(current_page):
                            for child_page in current_page.get_children():
                                translator.create_translations(child_page)

                                if child_page.numchild:
                                    _walk(child_page)

                        _walk(self.object)

                if len(form.cleaned_data["locales"]) == 1:
                    locales = form.cleaned_data["locales"][0].get_display_name()

                else:
                    # Note: always plural
                    locales = _('{} locales').format(len(form.cleaned_data["locales"]))

                # TODO: Button that links to page in translations report when we have it
                messages.success(
                    self.request, self.get_success_message(locales)
                )

                return redirect(self.get_success_url() or self.get_default_success_url())

        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)

    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perms(['wagtail_localize.submit_translation']):
            raise PermissionDenied

        self.object = self.get_object()
        return super().dispatch(request, *args, **kwargs)


class SubmitPageTranslationView(SubmitTranslationView):
    title = gettext_lazy("Translate page")

    def get_subtitle(self):
        return self.object.get_admin_display_title()

    def get_object(self):
        page = get_object_or_404(Page, id=self.kwargs['page_id']).specific

        # Can't translate the root page
        if page.is_root():
            raise Http404

        return page

    def get_default_success_url(self):
        return reverse("wagtailadmin_explore", args=[self.get_object().get_parent().id])

    def get_success_message(self, locales):
        return _("The page '{page_title}' was successfully submitted for translation into {locales}").format(
            page_title=self.object.get_admin_display_title(),
            locales=locales
        )


class SubmitSnippetTranslationView(SubmitTranslationView):

    def get_title(self):
        return _("Translate {model_name}").format(
            model_name=self.object._meta.verbose_name
        )

    def get_object(self):
        model = get_snippet_model_from_url_params(self.kwargs['app_label'], self.kwargs['model_name'])

        if not issubclass(model, TranslatableMixin):
            raise Http404

        return get_object_or_404(model, pk=unquote(self.kwargs['pk']))

    def get_default_success_url(self):
        return reverse("wagtailsnippets:edit", args=[self.kwargs['app_label'], self.kwargs['model_name'], self.kwargs['pk']])

    def get_success_message(self, locales):
        return _("The {model_name} '{object}' was successfully submitted for translation into {locales}").format(
            model_name=self.object._meta.verbose_name,
            object=str(self.object),
            locales=locales
        )



from django.db import models
from django.utils.translation import gettext_lazy

from modelcluster.fields import ParentalKey
from wagtail.admin.edit_handlers import TabbedInterface, ObjectList, FieldPanel, InlinePanel, PageChooserPanel
from wagtail.core import blocks
from wagtail.core.fields import RichTextField, StreamField
from wagtail.core.models import Page, Orderable, TranslatableMixin
from wagtail.documents.blocks import DocumentChooserBlock
from wagtail.embeds.blocks import EmbedBlock
from wagtail.images.blocks import ImageChooserBlock
from wagtail.snippets.blocks import SnippetChooserBlock
from wagtail.snippets.models import register_snippet

from wagtail_localize.fields import TranslatableField, SynchronizedField
from wagtail_localize.segments import StringSegmentValue


@register_snippet
class TestSnippet(TranslatableMixin, models.Model):
    field = models.TextField(gettext_lazy("field"))

    translatable_fields = [TranslatableField("field")]


@register_snippet
class NonTranslatableSnippet(models.Model):
    field = models.TextField()


class TestStructBlock(blocks.StructBlock):
    field_a = blocks.TextBlock()
    field_b = blocks.TextBlock()


class TestNestedStreamBlock(blocks.StreamBlock):
    block_a = blocks.TextBlock()
    block_b = blocks.TextBlock()


class CustomStructBlock(blocks.StructBlock):
    field_a = blocks.TextBlock()
    field_b = blocks.TextBlock()

    def get_translatable_segments(self, value):
        return [
            StringSegmentValue("foo", "{} / {}".format(value["field_a"], value["field_b"]))
        ]

    def restore_translated_segments(self, value, segments):
        for segment in segments:
            if segment.path == "foo":
                field_a, field_b = segment.render_text().split("/")
                value["field_a"] = field_a.strip()
                value["field_b"] = field_b.strip()

        return value


class CustomBlockWithoutExtractMethod(blocks.Block):
    pass


class TestStreamBlock(blocks.StreamBlock):
    test_charblock = blocks.CharBlock(max_length=255)
    test_textblock = blocks.TextBlock(label=gettext_lazy("text block"))
    test_emailblock = blocks.EmailBlock()
    test_urlblock = blocks.URLBlock()
    test_richtextblock = blocks.RichTextBlock()
    test_rawhtmlblock = blocks.RawHTMLBlock()
    test_blockquoteblock = blocks.BlockQuoteBlock()
    test_structblock = TestStructBlock()
    test_listblock = blocks.ListBlock(blocks.TextBlock())
    test_nestedstreamblock = TestNestedStreamBlock()
    test_customstructblock = CustomStructBlock()
    test_customblockwithoutextractmethod = CustomBlockWithoutExtractMethod()
    test_pagechooserblock = blocks.PageChooserBlock()
    test_pagechooserblock_with_restricted_types = blocks.PageChooserBlock(['wagtail_localize_test.TestHomePage', 'wagtail_localize_test.TestPage'])
    test_imagechooserblock = ImageChooserBlock()
    test_documentchooserblock = DocumentChooserBlock()
    test_snippetchooserblock = SnippetChooserBlock(TestSnippet)
    test_nontranslatablesnippetchooserblock = SnippetChooserBlock(NonTranslatableSnippet)
    test_embedblock = EmbedBlock()


class TestCustomField(models.TextField):
    def get_translatable_segments(self, value):
        if not value:
            # Don't disrupt other tests
            return []

        return [StringSegmentValue("foo", "{} and some extra".format(value))]


class TestPage(Page):
    test_charfield = models.CharField(gettext_lazy("char field"), max_length=255, blank=True, null=True, default='')
    test_textfield = models.TextField(blank=True)
    test_emailfield = models.EmailField(blank=True)
    test_slugfield = models.SlugField(blank=True)
    test_urlfield = models.URLField(blank=True)

    test_richtextfield = RichTextField(blank=True)
    test_streamfield = StreamField(TestStreamBlock, blank=True)

    test_snippet = models.ForeignKey(
        TestSnippet, null=True, blank=True, on_delete=models.SET_NULL
    )

    test_customfield = TestCustomField(blank=True)

    test_synchronized_charfield = models.CharField(max_length=255, blank=True)
    test_synchronized_textfield = models.TextField(blank=True)
    test_synchronized_emailfield = models.EmailField(blank=True)
    test_synchronized_slugfield = models.SlugField(blank=True)
    test_synchronized_urlfield = models.URLField(blank=True)

    test_synchronized_richtextfield = RichTextField(blank=True)
    test_synchronized_streamfield = StreamField(TestStreamBlock, blank=True)

    test_synchronized_image = models.ForeignKey(
        'wagtailimages.Image', null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )
    test_synchronized_document = models.ForeignKey(
        'wagtaildocs.Document', null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )
    test_synchronized_snippet = models.ForeignKey(
        TestSnippet, null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )

    test_page = models.ForeignKey(
        Page, null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )

    test_page_specific_type = models.ForeignKey(
        'TestHomePage', null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )

    test_page_with_restricted_types = models.ForeignKey(
        Page, null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )

    test_synchronized_customfield = TestCustomField(blank=True)

    translatable_fields = [
        TranslatableField("test_charfield"),
        TranslatableField("test_textfield"),
        TranslatableField("test_emailfield"),
        TranslatableField("test_slugfield"),
        TranslatableField("test_urlfield"),
        TranslatableField("test_richtextfield"),
        TranslatableField("test_streamfield"),
        TranslatableField("test_snippet"),
        TranslatableField("test_childobjects"),
        TranslatableField("test_customfield"),
        SynchronizedField("test_synchronized_charfield"),
        SynchronizedField("test_synchronized_textfield"),
        SynchronizedField("test_synchronized_emailfield"),
        SynchronizedField("test_synchronized_slugfield"),
        SynchronizedField("test_synchronized_urlfield"),
        SynchronizedField("test_synchronized_richtextfield"),
        SynchronizedField("test_synchronized_streamfield"),
        SynchronizedField("test_synchronized_image"),
        SynchronizedField("test_synchronized_document"),
        SynchronizedField("test_synchronized_snippet"),
        SynchronizedField("test_synchronized_childobjects"),
        SynchronizedField('test_page'),
        SynchronizedField('test_page_specific_type'),
        SynchronizedField('test_page_with_restricted_types'),
        SynchronizedField("test_synchronized_customfield"),
    ]

    content_panels = Page.content_panels + [
        FieldPanel("test_charfield"),
        FieldPanel("test_textfield"),
        FieldPanel("test_emailfield"),
        FieldPanel("test_slugfield"),
        FieldPanel("test_urlfield"),
        FieldPanel("test_richtextfield"),
        FieldPanel("test_streamfield"),
        FieldPanel("test_snippet"),
        InlinePanel("test_childobjects"),
        FieldPanel("test_customfield"),
        FieldPanel("test_synchronized_charfield"),
        FieldPanel("test_synchronized_textfield"),
        FieldPanel("test_synchronized_emailfield"),
        FieldPanel("test_synchronized_slugfield"),
        FieldPanel("test_synchronized_urlfield"),
        FieldPanel("test_synchronized_richtextfield"),
        FieldPanel("test_synchronized_streamfield"),
        FieldPanel("test_synchronized_image"),
        FieldPanel("test_synchronized_document"),
        FieldPanel("test_synchronized_snippet"),
        InlinePanel("test_synchronized_childobjects"),
        PageChooserPanel('test_page'),
        PageChooserPanel('test_page_specific_type'),
        PageChooserPanel('test_page_with_restricted_types', ['wagtail_localize_test.TestHomePage', 'wagtail_localize_test.TestPage']),
        FieldPanel("test_synchronized_customfield"),
    ]


class TestModel(TranslatableMixin):
    title = models.CharField(max_length=255)
    test_charfield = models.CharField(max_length=255, blank=True)
    test_textfield = models.TextField(blank=True)
    test_emailfield = models.EmailField(blank=True)

    translatable_fields = [
        TranslatableField("test_charfield"),
        TranslatableField("test_textfield"),
        TranslatableField("test_emailfield"),
    ]


class InheritedTestModel(TestModel):
    class Meta:
        unique_together = None


class TestChildObject(TranslatableMixin, Orderable):
    page = ParentalKey(TestPage, related_name="test_childobjects")
    field = models.TextField()

    translatable_fields = [TranslatableField("field")]

    class Meta(TranslatableMixin.Meta, Orderable.Meta):
        pass


# TODO: System check for TranslatableMixin here
class TestSynchronizedChildObject(Orderable):
    page = ParentalKey(TestPage, related_name="test_synchronized_childobjects")
    field = models.TextField()

    translatable_fields = [TranslatableField("field")]


# FIXME: Rename me
class TestNonParentalChildObject(TranslatableMixin, Orderable):
    page = models.ForeignKey(
        TestPage, on_delete=models.CASCADE, related_name="test_nonparentalchildobjects"  # FIXME: inconsistent related_name
    )
    field = models.TextField()

    translatable_fields = [TranslatableField("field")]


class TestHomePage(Page):
    pass


class TestGenerateTranslatableFieldsPage(Page):
    """
    A page type that tests the builtin automatic generation of translatable fields.
    """

    test_charfield = models.CharField(max_length=255, blank=True)
    test_charfield_with_choices = models.CharField(max_length=255, blank=True, choices=[('a', "A"), ('b', "B")])
    test_textfield = models.TextField(blank=True)
    test_emailfield = models.EmailField(blank=True)
    test_slugfield = models.SlugField(blank=True)
    test_urlfield = models.URLField(blank=True)

    test_richtextfield = RichTextField(blank=True)
    test_streamfield = StreamField(TestStreamBlock, blank=True)

    test_snippet = models.ForeignKey(
        TestSnippet, null=True, blank=True, on_delete=models.SET_NULL
    )

    test_nontranslatablesnippet = models.ForeignKey(
        NonTranslatableSnippet, null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )

    test_customfield = TestCustomField(blank=True)


class TestOverrideTranslatableFieldsPage(TestGenerateTranslatableFieldsPage):
    override_translatable_fields = [
        SynchronizedField('test_charfield'),
        TranslatableField('test_emailfield'),
    ]


class TranslatableChildObject(TranslatableMixin, Orderable):
    page = ParentalKey(TestGenerateTranslatableFieldsPage, related_name="test_translatable_childobjects")
    field = models.TextField()

    class Meta(TranslatableMixin.Meta, Orderable.Meta):
        pass


class NonTranslatableChildObject(Orderable):
    page = ParentalKey(TestGenerateTranslatableFieldsPage, related_name="test_nontranslatable_childobjects")
    field = models.TextField()


class TestModelWithInvalidForeignKey(TranslatableMixin, models.Model):
    fk = models.ForeignKey('wagtailcore.Site', on_delete=models.CASCADE)

    # This should raise an error as the model being pointed to is not
    # translatable!
    translatable_fields = [
        TranslatableField('fk'),
    ]


class PageWithCustomEditHandler(Page):
    foo_field = models.TextField()
    bar_field = models.TextField()
    baz_field = models.TextField()

    foo_panels = [
        FieldPanel('foo_field'),
    ]

    bar_panels = [
        FieldPanel('bar_field'),
        FieldPanel('baz_field'),
    ]

    edit_handler = TabbedInterface([
        ObjectList(bar_panels, heading="Bar"),
        ObjectList([InlinePanel('child_objects')], heading="Child objects"),
        ObjectList(foo_panels, heading="Foo"),
        ObjectList(Page.content_panels, heading="Content"),
        ObjectList(Page.promote_panels, heading="Promote"),
        ObjectList(Page.settings_panels, heading="Settings"),
    ])


class PageWithCustomEditHandlerChildObject(TranslatableMixin, Orderable):
    page = ParentalKey(PageWithCustomEditHandler, related_name="child_objects")
    field = models.TextField()

    class Meta(TranslatableMixin.Meta, Orderable.Meta):
        pass



import uuid
import unittest

from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase

from wagtail.core.blocks import StreamValue
from wagtail.core.models import Page, Site

from wagtail_localize.segments import (
    StringSegmentValue,
    TemplateSegmentValue,
    RelatedObjectSegmentValue,
    OverridableSegmentValue,
)
from wagtail_localize.segments.extract import extract_segments
from wagtail_localize.strings import StringValue
from wagtail_localize.test.models import (
    TestPage,
    TestSnippet,
    TestChildObject,
    TestNonParentalChildObject,
    TestModelWithInvalidForeignKey,
)


def make_test_page(**kwargs):
    root_page = Page.objects.get(id=1)
    kwargs.setdefault("title", "Test page")
    return root_page.add_child(instance=TestPage(**kwargs))


RICH_TEXT_TEST_INPUT = '<h1>This is a heading</h1><p>This is a paragraph. &lt;foo&gt; <b>Bold text</b></p><ul><li><a href="http://example.com">This is a link</a>.</li></ul>'

RICH_TEXT_TEST_OUTPUT = [
    TemplateSegmentValue(
        "",
        "html",
        '<h1><text position="0"></text></h1><p><text position="1"></text></p><ul><li><text position="2"></text></li></ul>',
        3,
    ),
    StringSegmentValue("", "This is a heading"),
    StringSegmentValue.from_source_html(
        "",
        'This is a paragraph. &lt;foo&gt; <b>Bold text</b>',
    ),
    StringSegmentValue(
        "",
        StringValue('<a id="a1">This is a link</a>.'),
        attrs={
            "a1": {"href": "http://example.com"}
        }
    ),
    OverridableSegmentValue(
        "'http://example.com'",
        "http://example.com"
    )
]


class TestSegmentExtraction(TestCase):
    def test_charfield(self):
        page = make_test_page(test_charfield="Test content")
        segments = extract_segments(page)

        self.assertEqual(segments, [StringSegmentValue("test_charfield", "Test content")])

    def test_null_charfield(self):
        page = make_test_page(test_charfield=None)
        segments = extract_segments(page)

        self.assertEqual(segments, [])

    def test_textfield(self):
        page = make_test_page(test_textfield="Test content")
        segments = extract_segments(page)

        self.assertEqual(segments, [StringSegmentValue("test_textfield", "Test content")])

    def test_emailfield(self):
        page = make_test_page(test_emailfield="test@example.com")
        segments = extract_segments(page)

        self.assertEqual(
            segments, [StringSegmentValue("test_emailfield", "test@example.com")]
        )

    def test_slugfield(self):
        page = make_test_page(test_slugfield="test-content")
        segments = extract_segments(page)

        self.assertEqual(segments, [StringSegmentValue("test_slugfield", "test-content")])

    def test_urlfield(self):
        page = make_test_page(test_urlfield="http://test-content.com/foo")
        segments = extract_segments(page)

        self.assertEqual(
            segments, [StringSegmentValue("test_urlfield", "http://test-content.com/foo")]
        )

    def test_richtextfield(self):
        page = make_test_page(test_richtextfield=RICH_TEXT_TEST_INPUT)
        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [segment.wrap("test_richtextfield") for segment in RICH_TEXT_TEST_OUTPUT],
        )

    def test_snippet(self):
        test_snippet = TestSnippet.objects.create(field="Test content")
        page = make_test_page(test_snippet=test_snippet)
        segments = extract_segments(page)

        self.assertEqual(
            segments, [RelatedObjectSegmentValue.from_instance("test_snippet", test_snippet)]
        )

    def test_childobjects(self):
        page = make_test_page()
        page.test_childobjects.add(TestChildObject(field="Test content"))
        page.save()

        child_translation_key = TestChildObject.objects.get().translation_key

        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [
                StringSegmentValue(
                    f"test_childobjects.{child_translation_key}.field", "Test content"
                )
            ],
        )

    def test_nonparentalchildobjects(self):
        page = make_test_page()
        page.save()
        TestNonParentalChildObject.objects.create(page=page, field="Test content")

        segments = extract_segments(page)

        # No segments this time as we don't extract ManyToOneRel's that don't use ParentalKeys
        self.assertEqual(segments, [])

    def test_customfield(self):
        page = make_test_page(test_customfield="Test content")

        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [StringSegmentValue("test_customfield.foo", "Test content and some extra")],
        )


def make_test_page_with_streamfield_block(block_id, block_type, block_value, **kwargs):
    stream_data = [{"id": block_id, "type": block_type, "value": block_value}]

    return make_test_page(
        test_streamfield=StreamValue(
            TestPage.test_streamfield.field.stream_block, stream_data, is_lazy=True
        ),
        **kwargs,
    )


class TestSegmentExtractionWithStreamField(TestCase):
    def test_charblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_charblock", "Test content"
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments, [StringSegmentValue(f"test_streamfield.{block_id}", "Test content")]
        )

    def test_textblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_textblock", "Test content"
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments, [StringSegmentValue(f"test_streamfield.{block_id}", "Test content")]
        )

    @unittest.expectedFailure  # Not supported
    def test_emailblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_emailblock", "test@example.com"
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments, [StringSegmentValue(f"test_streamfield.{block_id}", "test@example.com")]
        )

    @unittest.expectedFailure  # Not supported
    def test_urlblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_urlblock", "http://test-content.com/foo"
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [
                StringSegmentValue(
                    f"test_streamfield.{block_id}", "http://test-content.com/foo"
                )
            ],
        )

    def test_embedblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_embedblock", "https://www.youtube.com/watch?v=aBByJQCaaEA"
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [
                OverridableSegmentValue(
                    f"test_streamfield.{block_id}", "https://www.youtube.com/watch?v=aBByJQCaaEA"
                )
            ],
        )

    def test_richtextblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_richtextblock", RICH_TEXT_TEST_INPUT
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [
                segment.wrap(f"test_streamfield.{block_id}")
                for segment in RICH_TEXT_TEST_OUTPUT
            ],
        )

    @unittest.expectedFailure  # Not supported
    def test_rawhtmlblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_rawhtmlblock", RICH_TEXT_TEST_INPUT
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [
                segment.wrap(f"test_streamfield.{block_id}")
                for segment in RICH_TEXT_TEST_OUTPUT
            ],
        )

    def test_blockquoteblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_blockquoteblock", "Test content"
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments, [StringSegmentValue(f"test_streamfield.{block_id}", "Test content")]
        )

    def test_structblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id),
            "test_structblock",
            {"field_a": "Test content", "field_b": "Some more test content"},
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [
                StringSegmentValue(f"test_streamfield.{block_id}.field_a", "Test content"),
                StringSegmentValue(
                    f"test_streamfield.{block_id}.field_b", "Some more test content"
                ),
            ],
        )

    @unittest.expectedFailure  # Not supported (probably won't ever be due to lack of path stability)
    def test_listblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_listblock", ["Test content", "Some more test content"]
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [
                StringSegmentValue(f"test_streamfield.{block_id}", "Test content"),
                StringSegmentValue(f"test_streamfield.{block_id}", "Some more test content"),
            ],
        )

    def test_nestedstreamblock(self):
        block_id = uuid.uuid4()
        nested_block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id),
            "test_nestedstreamblock",
            [{"id": str(nested_block_id), "type": "block_a", "value": "Test content"}],
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [
                StringSegmentValue(
                    f"test_streamfield.{block_id}.{nested_block_id}", "Test content"
                )
            ],
        )

    def test_customstructblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id),
            "test_customstructblock",
            {"field_a": "Test content", "field_b": "Some more test content"},
        )

        segments = extract_segments(page)

        self.assertEqual(
            segments,
            [
                StringSegmentValue(
                    f"test_streamfield.{block_id}.foo",
                    "Test content / Some more test content",
                )
            ],
        )

    def test_customblockwithoutextractmethod(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id),
            "test_customblockwithoutextractmethod",
            {},
        )

        segments = extract_segments(page)
        self.assertEqual(segments, [])

    def test_invalid_foreign_key(self):
        instance = TestModelWithInvalidForeignKey.objects.create(fk=Site.objects.get())

        with self.assertRaises(ImproperlyConfigured) as e:
            extract_segments(instance)

        self.assertEqual(
            str(e.exception),
            "The foreign key `wagtail_localize_test.TestModelWithInvalidForeignKey.fk` "
            "was registered as a translatable field but the model it points to "
            "`wagtailcore.Site` is not translatable"
        )



"""
Django settings for temp project.

Generated by 'django-admin startproject' using Django 1.10.5.

For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

import os
import dj_database_url

# Build paths inside the project like this: os.path.join(PROJECT_DIR, ...)
PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
BASE_DIR = os.path.dirname(PROJECT_DIR)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "c6u0-9c!7nilj_ysatsda0(f@e_2mws2f!6m0n^o*4#*q#kzp)"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["localhost", "testserver"]


# Application definition

INSTALLED_APPS = [
    "wagtail_localize",
    "wagtail_localize.locales",
    "wagtail_localize.test",
    "wagtail.contrib.search_promotions",
    "wagtail.contrib.forms",
    "wagtail.contrib.redirects",
    "wagtail.embeds",
    "wagtail.users",
    "wagtail.snippets",
    "wagtail.documents",
    "wagtail.images",
    "wagtail.search",
    "wagtail.admin",
    "wagtail.api.v2",
    "wagtail.contrib.modeladmin",
    "wagtail.contrib.routable_page",
    "wagtail.sites",
    "wagtail.core",
    "taggit",
    "rest_framework",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.sitemaps",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "wagtail.contrib.redirects.middleware.RedirectMiddleware",
]

ROOT_URLCONF = "wagtail_localize.test.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]


# Using DatabaseCache to make sure that the cache is cleared between tests.
# This prevents false-positives in some wagtail core tests where we are
# changing the 'wagtail_root_paths' key which may cause future tests to fail.
CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.db.DatabaseCache",
        "LOCATION": "cache",
    }
}


# don't use the intentionally slow default password hasher
PASSWORD_HASHERS = ("django.contrib.auth.hashers.MD5PasswordHasher",)


# Database
# https://docs.djangoproject.com/en/1.10/ref/settings/#databases

DATABASES = {
    "default": dj_database_url.config(default="sqlite:///test_wagtaillocalize.db"),
}


# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
    },
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
    {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
]


# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.8/howto/static-files/

STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]

STATICFILES_DIRS = [os.path.join(PROJECT_DIR, "static")]

STATIC_ROOT = os.path.join(PROJECT_DIR, "collect_static")
STATIC_URL = "/static/"

MEDIA_ROOT = os.path.join(BASE_DIR, "test-media")
MEDIA_URL = "/media/"


# Wagtail settings
WAGTAIL_SITE_NAME = "Wagtail localize test project"

WAGTAIL_I18N_ENABLED = True

LANGUAGES = WAGTAIL_CONTENT_LANGUAGES = [("en", "English"), ("fr", "French"), ("fr-CA", "French (Canada)"), ("es", "Spanish")]

WAGTAILLOCALIZE_MACHINE_TRANSLATOR = {
    'CLASS': 'wagtail_localize.machine_translators.dummy.DummyTranslator',
}



from django.conf import settings
from django.utils.module_loading import import_string


def get_machine_translator():
    config = getattr(settings, 'WAGTAILLOCALIZE_MACHINE_TRANSLATOR', None)

    if config is None:
        return

    # Raises ImportError
    machine_translator_class = import_string(config['CLASS'])

    return machine_translator_class(config.get('OPTIONS', {}))



from django.contrib.auth.models import Permission
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from wagtail.admin.menu import MenuItem
from wagtail.core import hooks
from wagtail.core.permissions import site_permission_policy

from .views import LocaleViewSet


@hooks.register('register_admin_viewset')
def register_viewset():
    return LocaleViewSet('wagtaillocales', url_prefix='locales')


class LocalesMenuItem(MenuItem):
    def is_shown(self, request):
        return site_permission_policy.user_has_any_permission(
            request.user, ['add', 'change', 'delete']
        )


@hooks.register('register_settings_menu_item')
def register_locales_menu_item():
    return LocalesMenuItem(_('Locales'), reverse('wagtaillocales:index'),
                           icon_name='site', order=603)


@hooks.register('register_permissions')
def register_permissions():
    return Permission.objects.filter(content_type__app_label='wagtailcore',
                                     codename__in=['add_locale', 'change_locale', 'delete_locale'])



from django.test import TestCase

from wagtail_localize.fields import get_translatable_fields, SynchronizedField, TranslatableField

from wagtail_localize.test.models import TestGenerateTranslatableFieldsPage, TestOverrideTranslatableFieldsPage


class TestGetTranslatableFields(TestCase):
    def test(self):
        translatable_fields = get_translatable_fields(TestGenerateTranslatableFieldsPage)

        self.assertEqual(translatable_fields, [
            TranslatableField('title'),
            TranslatableField('slug'),
            TranslatableField('seo_title'),
            SynchronizedField('show_in_menus'),
            TranslatableField('search_description'),
            TranslatableField('test_charfield'),
            SynchronizedField('test_charfield_with_choices'),
            TranslatableField('test_textfield'),
            SynchronizedField('test_emailfield'),
            TranslatableField('test_slugfield'),
            SynchronizedField('test_urlfield'),
            TranslatableField('test_richtextfield'),
            TranslatableField('test_streamfield'),
            TranslatableField('test_snippet'),
            SynchronizedField('test_nontranslatablesnippet'),
            TranslatableField('test_customfield'),
            TranslatableField('test_translatable_childobjects'),
            SynchronizedField('test_nontranslatable_childobjects'),
        ])


class TestOverrideTranslatableFields(TestCase):
    def test(self):
        translatable_fields = get_translatable_fields(TestOverrideTranslatableFieldsPage)

        self.assertEqual(translatable_fields, [
            TranslatableField('title'),
            TranslatableField('slug'),
            TranslatableField('seo_title'),
            SynchronizedField('show_in_menus'),
            TranslatableField('search_description'),
            SynchronizedField('test_charfield'),  # Overriden!
            SynchronizedField('test_charfield_with_choices'),
            TranslatableField('test_textfield'),
            TranslatableField('test_emailfield'),  # Overriden!
            TranslatableField('test_slugfield'),
            SynchronizedField('test_urlfield'),
            TranslatableField('test_richtextfield'),
            TranslatableField('test_streamfield'),
            TranslatableField('test_snippet'),
            SynchronizedField('test_nontranslatablesnippet'),
            TranslatableField('test_customfield'),
            TranslatableField('test_translatable_childobjects'),
            SynchronizedField('test_nontranslatable_childobjects'),
        ])



from django.test import TestCase

from wagtail_localize.strings import (
    StringValue,
    extract_strings,
    restore_strings,
    extract_ids,
    validate_translation_links,
)


class TestStringValueFromSourceHTML(TestCase):
    def test_string_from_html(self):
        string, attrs = StringValue.from_source_html(
            '<b>Bread</b>\xa0is a\xa0<a href="https://en.wikipedia.org/wiki/Staple_food">staple food</a>\xa0prepared from a\xa0<a href="https://en.wikipedia.org/wiki/Dough">dough</a>\xa0of\xa0<a href="https://en.wikipedia.org/wiki/Flour">flour</a>\xa0and\xa0<a href="https://en.wikipedia.org/wiki/Water">water</a>'
        )

        self.assertEqual(
            string.data,
            '<b>Bread</b> is a <a id="a1">staple food</a> prepared from a <a id="a2">dough</a> of <a id="a3">flour</a> and <a id="a4">water</a>',
        )

        self.assertEqual(
            attrs,
            {
                'a1': {'href': 'https://en.wikipedia.org/wiki/Staple_food'},
                'a2': {'href': 'https://en.wikipedia.org/wiki/Dough'},
                'a3': {'href': 'https://en.wikipedia.org/wiki/Flour'},
                'a4': {'href': 'https://en.wikipedia.org/wiki/Water'},
            }
        )

    def test_validation(self):
        # All of these should be allowed
        StringValue.from_source_html("Foo bar baz")
        StringValue.from_source_html("This is a paragraph. <b>This is some bold <i>and now italic</i></b> text")
        StringValue.from_source_html("&lt;script&gt; this should be interpreted as text.")
        StringValue.from_source_html("Foo<br/>bar<br/>baz")
        StringValue.from_source_html('<a href="https://en.wikipedia.org/wiki/Staple_food">staple food</a>')

    def test_block_tags_not_allowed(self):
        with self.assertRaises(ValueError) as e:
            StringValue.from_source_html("<p>Foo bar baz</p>")

        self.assertIsInstance(e.exception, ValueError)
        self.assertEqual(e.exception.args, ('<p> tag is not allowed. Strings can only contain standard HTML inline tags (such as <b>, <a>)',))

        with self.assertRaises(ValueError) as e:
            StringValue.from_source_html("<img/>")

        self.assertIsInstance(e.exception, ValueError)
        self.assertEqual(e.exception.args, ('<img> tag is not allowed. Strings can only contain standard HTML inline tags (such as <b>, <a>)',))


class TestStringValueFromTranslatedHTML(TestCase):
    def test_validation(self):
        # All of these should be allowed
        StringValue.from_translated_html("Foo bar baz")
        StringValue.from_translated_html("This is a paragraph. <b>This is some bold <i>and now italic</i></b> text")
        StringValue.from_translated_html("&lt;script&gt; this should be interpreted as text.")
        StringValue.from_translated_html("Foo<br/>bar<br/>baz")
        StringValue.from_translated_html('<a id="1">staple food</a>')

    def test_block_tags_not_allowed(self):
        with self.assertRaises(ValueError) as e:
            StringValue.from_translated_html("<p>Foo bar baz</p>")

        self.assertIsInstance(e.exception, ValueError)
        self.assertEqual(e.exception.args, ('<p> tag is not allowed. Strings can only contain standard HTML inline tags (such as <b>, <a>)',))

        with self.assertRaises(ValueError) as e:
            StringValue.from_translated_html("<img/>")

        self.assertIsInstance(e.exception, ValueError)
        self.assertEqual(e.exception.args, ('<img> tag is not allowed. Strings can only contain standard HTML inline tags (such as <b>, <a>)',))

    def test_attributes_not_allowed(self):
        with self.assertRaises(ValueError) as e:
            StringValue.from_translated_html('<a href="https://en.wikipedia.org/wiki/Staple_food">staple food</a>')

        self.assertIsInstance(e.exception, ValueError)
        self.assertEqual(e.exception.args, ("Strings cannot have any HTML tags with attributes (except for 'id' in <a> tags)",))


class TestStringValueFromPlaintext(TestCase):
    def test_string_from_plaintext(self):
        string = StringValue.from_plaintext(
            "This is a test, \"foo\" bar 'baz'.!?;:",
        )

        self.assertEqual(
            string.data,
            "This is a test, \"foo\" bar 'baz'.!?;:",
        )

    def test_special_chars_escaped(self):
        string = StringValue.from_plaintext("<Foo> & bar")

        self.assertEqual(string.data, "&lt;Foo&gt; &amp; bar")

    def test_unicode_chars_not_escaped(self):
        # unicode characters should not be escaped as this would be horrible for translators!
        string = StringValue.from_plaintext("セキレイ")
        self.assertEqual(string.data, "セキレイ")

    def test_newlines_converted_to_br_tags(self):
        string = StringValue.from_plaintext("foo\nbar\nbaz")
        self.assertEqual(string.data, "foo<br/>bar<br/>baz")

        string = StringValue.from_plaintext("\nfoo\nbar\n")
        self.assertEqual(string.data, "<br/>foo<br/>bar<br/>")


class TestRenderHTML(TestCase):
    maxDiff = None

    def test_render_html(self):
        # Note, I swapped the first two links around to check that the attrs are restored to the correct one
        string = StringValue(
            '<b>Bread</b> is a <a id="a2">dough</a> prepared from a <a id="a1">staple food</a> of <a id="a3">flour</a> and <a id="a4">water</a>',
        )

        attrs = {
            'a1': {'href': 'https://en.wikipedia.org/wiki/Staple_food'},
            'a2': {'href': 'https://en.wikipedia.org/wiki/Dough'},
            'a3': {'href': 'https://en.wikipedia.org/wiki/Flour'},
            'a4': {'href': 'https://en.wikipedia.org/wiki/Water'},
        }

        self.assertEqual(
            string.render_html(attrs),
            '<b>Bread</b> is a <a href="https://en.wikipedia.org/wiki/Dough">dough</a> prepared from a <a href="https://en.wikipedia.org/wiki/Staple_food">staple food</a> of <a href="https://en.wikipedia.org/wiki/Flour">flour</a> and <a href="https://en.wikipedia.org/wiki/Water">water</a>',
        )


class TestStringRenderText(TestCase):
    def test_string_render_text(self):
        string = StringValue(
            "This is a paragraph. <b>This is some bold <i>and now italic</i></b> text"
        )

        self.assertEqual(
            string.render_text(), "This is a paragraph. This is some bold and now italic text"
        )

    def test_special_chars_unescaped(self):
        string = StringValue("<b>foo</b><i> &amp; bar</i>")
        self.assertEqual(string.render_text(), "foo & bar")

    def test_br_tags_converted_to_newlines(self):
        string = StringValue("foo<br>bar<br>baz")
        self.assertEqual(string.render_text(), "foo\nbar\nbaz")

        string = StringValue("<br/><b>foo</b><br/><i>bar</i><br/>")
        self.assertEqual(string.render_text(), "\nfoo\nbar\n")


class TextExtractStrings(TestCase):
    def test_extract_strings(self):
        template, strings = extract_strings(
            """
            <p><b>Bread</b>\xa0is a\xa0<a href="https://en.wikipedia.org/wiki/Staple_food">staple food</a>\xa0prepared from a\xa0<a href="https://en.wikipedia.org/wiki/Dough">dough</a>\xa0of\xa0<a href="https://en.wikipedia.org/wiki/Flour">flour</a>\xa0and\xa0<a href="https://en.wikipedia.org/wiki/Water">water</a>, usually by\xa0<a href="https://en.wikipedia.org/wiki/Baking">baking</a>. Throughout recorded history it has been popular around the world and is one of the oldest artificial foods, having been of importance since the dawn of\xa0<a href="https://en.wikipedia.org/wiki/Agriculture#History">agriculture</a>.</p>
            <p>Proportions of types of flour and other ingredients vary widely, as do modes of preparation. As a result, types, shapes, sizes, and textures of breads differ around the world. Bread may be\xa0<a href="https://en.wikipedia.org/wiki/Leaven">leavened</a>\xa0by processes such as reliance on naturally occurring\xa0<a href="https://en.wikipedia.org/wiki/Sourdough">sourdough</a>\xa0microbes, chemicals, industrially produced yeast, or high-pressure aeration. Some bread is cooked before it can leaven, including for traditional or religious reasons. Non-cereal ingredients such as fruits, nuts and fats may be included. Commercial bread commonly contains additives to improve flavor, texture, color, shelf life, and ease of manufacturing.</p>
            """
        )

        self.assertHTMLEqual(
            template,
            """
            <p><text position="0"></text></p>
            <p><text position="1"></text></p>
            """,
        )

        self.assertEqual(
            strings,
            [
                StringValue.from_source_html('<b>Bread</b>\xa0is a\xa0<a href="https://en.wikipedia.org/wiki/Staple_food">staple food</a>\xa0prepared from a\xa0<a href="https://en.wikipedia.org/wiki/Dough">dough</a>\xa0of\xa0<a href="https://en.wikipedia.org/wiki/Flour">flour</a>\xa0and\xa0<a href="https://en.wikipedia.org/wiki/Water">water</a>, usually by\xa0<a href="https://en.wikipedia.org/wiki/Baking">baking</a>. Throughout recorded history it has been popular around the world and is one of the oldest artificial foods, having been of importance since the dawn of\xa0<a href="https://en.wikipedia.org/wiki/Agriculture#History">agriculture</a>.'),
                StringValue.from_source_html('Proportions of types of flour and other ingredients vary widely, as do modes of preparation. As a result, types, shapes, sizes, and textures of breads differ around the world. Bread may be\xa0<a href="https://en.wikipedia.org/wiki/Leaven">leavened</a>\xa0by processes such as reliance on naturally occurring\xa0<a href="https://en.wikipedia.org/wiki/Sourdough">sourdough</a>\xa0microbes, chemicals, industrially produced yeast, or high-pressure aeration. Some bread is cooked before it can leaven, including for traditional or religious reasons. Non-cereal ingredients such as fruits, nuts and fats may be included. Commercial bread commonly contains additives to improve flavor, texture, color, shelf life, and ease of manufacturing.'),
            ],
        )

    def test_extract_strings_2(self):
        template, strings = extract_strings(
            """
            <h1>Foo bar baz</h1>
            <p>This is a paragraph. <b>This is some bold <i>and now italic</i></b> text</p>
            <p>&lt;script&gt; this should be interpreted as text.</p>
            <ul>
                <li>List item one</li>
                <li><b>List item two</li>
            </ul>
            <img src="foo" alt="This bit isn't translatable">
            """
        )

        self.assertHTMLEqual(
            template,
            """
            <h1><text position="0"></text></h1>
            <p><text position="1"></text></p>
            <p><text position="2"></text></p>
            <ul>
                <li><text position="3"></text></li>
                <li><b><text position="4"></text></b></li>
            </ul>
            <img alt="This bit isn\'t translatable" src="foo">
            """,
        )

        self.assertEqual(
            strings,
            [
                StringValue.from_source_html("Foo bar baz"),
                StringValue.from_source_html("This is a paragraph. <b>This is some bold <i>and now italic</i></b> text"),
                StringValue.from_source_html("&lt;script&gt; this should be interpreted as text."),
                StringValue.from_source_html("List item one"),
                StringValue.from_source_html("List item two"),
            ],
        )

    def test_block_tag_in_inline_tag(self):
        # If an inline tag contains a block tag. The inline tag must be in the template.
        # Testing for issue https://github.com/mozilla/donate-wagtail/issues/586
        template, strings = extract_strings("<p><i>Foo <p>Bar</p></i></p>")

        self.assertHTMLEqual(
            template,
            '<p><i><text position="0"></text> <p><text position="1"></text></p></i></p>',
        )

        self.assertEqual(strings, [
            StringValue.from_source_html("Foo"),
            StringValue.from_source_html("Bar")
        ])

    def test_br_tag_is_treated_as_inline_tag(self):
        template, strings = extract_strings(
            "<p><b>Foo <i>Bar<br/>Baz</i></b></p>"
        )

        self.assertHTMLEqual(template, '<p><b><text position="0"></text></b></p>')

        self.assertEqual(strings, [
            StringValue.from_source_html("Foo <i>Bar<br/>Baz</i>")
        ])

    def test_br_tag_is_removed_when_it_appears_at_beginning_of_segment(self):
        template, strings = extract_strings("<p><i><br/>Foo</i></p>")

        self.assertHTMLEqual(template, '<p><i><br/><text position="0"></text></i></p>')

        self.assertEqual(strings, [StringValue.from_source_html("Foo")])

    def test_br_tag_is_removed_when_it_appears_at_end_of_segment(self):
        template, strings = extract_strings("<p><i>Foo</i><br/></p>")

        self.assertHTMLEqual(template, '<p><i><text position="0"></text></i><br/></p>')

        self.assertEqual(strings, [StringValue.from_source_html("Foo")])

    def test_empty_inline_tag(self):
        template, strings = extract_strings("<p><i></i>Foo</p>")

        self.assertHTMLEqual(template, '<p><i></i><text position="0"></text></p>')

        self.assertEqual(strings, [StringValue.from_source_html("Foo")])


class TestRestoreStrings(TestCase):
    def test_restore_strings(self):
        html = restore_strings(
            """
            <p><text position="0"></text></p>
            <p><text position="1"></text></p>
            """,
            [
                StringValue.from_source_html('<b>Bread</b>\xa0is a\xa0<a href="https://en.wikipedia.org/wiki/Staple_food">staple food</a>\xa0prepared from a\xa0<a href="https://en.wikipedia.org/wiki/Dough">dough</a>\xa0of\xa0<a href="https://en.wikipedia.org/wiki/Flour">flour</a>\xa0and\xa0<a href="https://en.wikipedia.org/wiki/Water">water</a>, usually by\xa0<a href="https://en.wikipedia.org/wiki/Baking">baking</a>. Throughout recorded history it has been popular around the world and is one of the oldest artificial foods, having been of importance since the dawn of\xa0<a href="https://en.wikipedia.org/wiki/Agriculture#History">agriculture</a>.'),
                StringValue.from_source_html('Proportions of types of flour and other ingredients vary widely, as do modes of preparation. As a result, types, shapes, sizes, and textures of breads differ around the world. Bread may be\xa0<a href="https://en.wikipedia.org/wiki/Leaven">leavened</a>\xa0by processes such as reliance on naturally occurring\xa0<a href="https://en.wikipedia.org/wiki/Sourdough">sourdough</a>\xa0microbes, chemicals, industrially produced yeast, or high-pressure aeration. Some bread is cooked before it can leaven, including for traditional or religious reasons. Non-cereal ingredients such as fruits, nuts and fats may be included. Commercial bread commonly contains additives to improve flavor, texture, color, shelf life, and ease of manufacturing.'),
            ],
        )

        self.assertHTMLEqual(
            """
            <p><b>Bread</b>\xa0is a\xa0<a href="https://en.wikipedia.org/wiki/Staple_food">staple food</a>\xa0prepared from a\xa0<a href="https://en.wikipedia.org/wiki/Dough">dough</a>\xa0of\xa0<a href="https://en.wikipedia.org/wiki/Flour">flour</a>\xa0and\xa0<a href="https://en.wikipedia.org/wiki/Water">water</a>, usually by\xa0<a href="https://en.wikipedia.org/wiki/Baking">baking</a>. Throughout recorded history it has been popular around the world and is one of the oldest artificial foods, having been of importance since the dawn of\xa0<a href="https://en.wikipedia.org/wiki/Agriculture#History">agriculture</a>.</p>
            <p>Proportions of types of flour and other ingredients vary widely, as do modes of preparation. As a result, types, shapes, sizes, and textures of breads differ around the world. Bread may be\xa0<a href="https://en.wikipedia.org/wiki/Leaven">leavened</a>\xa0by processes such as reliance on naturally occurring\xa0<a href="https://en.wikipedia.org/wiki/Sourdough">sourdough</a>\xa0microbes, chemicals, industrially produced yeast, or high-pressure aeration. Some bread is cooked before it can leaven, including for traditional or religious reasons. Non-cereal ingredients such as fruits, nuts and fats may be included. Commercial bread commonly contains additives to improve flavor, texture, color, shelf life, and ease of manufacturing.</p>
            """,
            html,
        )

    def test_restore_strings_2(self):
        html = restore_strings(
            """
            <h1><text position="0"></text></h1>
            <p><text position="1"></text></p>
            <p><text position="2"></text></p>
            <ul>
                <li><text position="3"></text></li>
                <li><text position="4"></text></li>
            </ul>
            <img alt="This bit isn\'t translatable" src="foo">
            """,
            [
                StringValue.from_source_html("Foo bar baz"),
                StringValue.from_source_html("This is a paragraph. <b>This is some bold <i>and now italic</i></b> text"),
                StringValue.from_source_html("&lt;script&gt; this should be interpreted as text."),
                StringValue.from_source_html("List item one"),
                StringValue.from_source_html("<b>List item two</b>"),
            ],
        )

        self.assertHTMLEqual(
            """
            <h1>Foo bar baz</h1>
            <p>This is a paragraph. <b>This is some bold <i>and now italic</i></b> text</p>
            <p>&lt;script&gt; this should be interpreted as text.</p>
            <ul>
                <li>List item one</li>
                <li><b>List item two</li>
            </ul>
            <img src="foo" alt="This bit isn't translatable">
            """,
            html,
        )


class IDsValidationTestCase(TestCase):
    def test_extract_ids(self):
        ids = extract_ids(
            """
            <a id="a1">link1</a>
            <strong>
                <a id="a2">link2</a>
                <a id="a1">link1</a>
                <em>
                    <a id="a3">link3</a>
                </em>
            <strong>
            """
        )
        self.assertEqual({"a1", "a2", "a3"}, ids)

    def test_validate_translation_links_ok(self):
        validate_translation_links(
            """
            <a id="a1">link1</a>
            <strong>
                <a id="a2">link2</a>
            <strong>
            """,
            """
            <a id="a1">lien 1</a>
            <strong>
                <a id="a2">lien 2</a>
            <strong>
            """
        )

    def test_validate_translation_links_raises_error_if_new_ids(self):
        with self.assertRaises(ValueError) as e:
            validate_translation_links(
                """
                <a id="a1">link1</a>
                <strong>
                    <a id="a2">link2</a>
                <strong>
                """,
                """
                <a id="a1">lien 1</a>
                <strong>
                    <a id="a2">lien 2</a>
                    <a id="a3">lien 3</a>
                    <a id="a4">lien 4</a>
                <strong>
                """
            )
        self.assertEqual(e.exception.args, ("Unrecognised id found in an <a> tag: a3, a4",))



# Generated by Django 3.0.8 on 2020-08-05 09:04

from django.db import migrations
from django.db.models import OuterRef, Subquery


def delete_translation_sources(apps, schema_editor):
    # Delete all but the latest source for each object

    TranslatableObject = apps.get_model('wagtail_localize.TranslatableObject')
    TranslationSource = apps.get_model('wagtail_localize.TranslationSource')

    lastest_sources = TranslationSource.objects.filter(object_id=OuterRef('translation_key')).order_by('-created_at').values('id')[:1]
    sources_to_keep = TranslatableObject.objects.annotate(
        latest_source_id=Subquery(lastest_sources)
    ).values_list('latest_source_id', flat=True)

    TranslationSource.objects.exclude(id__in=sources_to_keep).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize', '0002_translation'),
    ]

    operations = [
        migrations.RunPython(delete_translation_sources, migrations.RunPython.noop),
    ]



from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase, override_settings
from django.urls import reverse
from wagtail.core.models import Page, Locale
from wagtail.tests.utils import WagtailTestUtils

from wagtail_localize.models import Translation, TranslationSource, StringSegment
from wagtail_localize.test.models import TestPage, TestSnippet, NonTranslatableSnippet

from .utils import assert_permission_denied


def make_test_page(parent, cls=None, **kwargs):
    cls = cls or TestPage
    kwargs.setdefault("title", "Test page")
    return parent.add_child(instance=cls(**kwargs))


def strip_user_perms():
    """
    Removes user permissions so they can still access admin and edit pages but can't submit anything for translation.
    """
    editors_group = Group.objects.get(name="Editors")
    editors_group.permissions.filter(codename='submit_translation').delete()

    for permission in Permission.objects.filter(content_type=ContentType.objects.get_for_model(TestSnippet)):
        editors_group.permissions.add(permission)

    for permission in Permission.objects.filter(content_type=ContentType.objects.get_for_model(NonTranslatableSnippet)):
        editors_group.permissions.add(permission)

    user = get_user_model().objects.get()
    user.is_superuser = False
    user.groups.add(editors_group)
    user.save()


@override_settings(
    LANGUAGES=[
        ('en', "English"),
        ('fr', "French"),
        ('de', "German"),
        ('es', "Spanish"),
    ],
    WAGTAIL_CONTENT_LANGUAGES=[
        ('en', "English"),
        ('fr', "French"),
        ('de', "German"),
        ('es', "Spanish"),
    ],
)
class TestPageUpdateTranslationsListingButton(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

        self.en_locale = Locale.objects.get()
        self.fr_locale = Locale.objects.create(language_code="fr")
        self.de_locale = Locale.objects.create(language_code="de")

        self.en_homepage = Page.objects.get(depth=2)
        self.fr_homepage = self.en_homepage.copy_for_translation(self.fr_locale)
        self.de_homepage = self.en_homepage.copy_for_translation(self.de_locale)

        self.en_blog_index = make_test_page(self.en_homepage, title="Blog", slug="blog")

        self.source, created = TranslationSource.get_or_create_from_instance(self.en_blog_index)
        self.translation = Translation.objects.create(
            source=self.source,
            target_locale=self.fr_locale
        )

    def test(self):
        response = self.client.get(
            reverse("wagtailadmin_explore", args=[self.en_homepage.id])
        )

        self.assertContains(response, (
            f'<a href="/admin/localize/update/{self.source.id}/?next=%2Fadmin%2Fpages%2F{self.en_homepage.id}%2F" aria-label="" class="u-link is-live ">\n'
            '                    Sync translated pages\n                </a>'
        ))

    def test_hides_if_page_hasnt_got_translations(self):
        self.source.delete()

        response = self.client.get(
            reverse("wagtailadmin_explore", args=[self.en_homepage.id])
        )

        self.assertNotContains(response, "Sync translated pages")

    def test_hides_if_user_doesnt_have_permission(self):
        strip_user_perms()

        response = self.client.get(
            reverse("wagtailadmin_explore", args=[self.en_homepage.id])
        )

        self.assertNotContains(response, "Sync translated pages")


@override_settings(
    LANGUAGES=[
        ('en', "English"),
        ('fr', "French"),
        ('de', "German"),
        ('es', "Spanish"),
    ],
    WAGTAIL_CONTENT_LANGUAGES=[
        ('en', "English"),
        ('fr', "French"),
        ('de', "German"),
        ('es', "Spanish"),
    ],
)
class TestSnippetUpdateTranslationsListingButton(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

        self.en_locale = Locale.objects.get()
        self.fr_locale = Locale.objects.create(language_code="fr")
        self.de_locale = Locale.objects.create(language_code="de")

        self.en_snippet = TestSnippet.objects.create(field="Test snippet")
        self.fr_snippet = self.en_snippet.copy_for_translation(self.fr_locale)
        self.fr_snippet.save()

        self.source, created = TranslationSource.get_or_create_from_instance(self.en_snippet)
        self.translation = Translation.objects.create(
            source=self.source,
            target_locale=self.fr_locale
        )

    def test(self):
        response = self.client.get(
            reverse("wagtailsnippets:list", args=['wagtail_localize_test', 'testsnippet'])
        )

        self.assertContains(response, (
            f'href="/admin/localize/update/{self.source.id}/?next=%2Fadmin%2Fsnippets%2Fwagtail_localize_test%2Ftestsnippet%2F">Sync translated snippets</a>'
        ))

    def test_hides_if_snippet_hasnt_got_translations(self):
        self.source.delete()

        response = self.client.get(
            reverse("wagtailsnippets:list", args=['wagtail_localize_test', 'testsnippet'])
        )

        self.assertNotContains(response, "Sync translated snippets")

    def test_hides_if_user_doesnt_have_permission(self):
        strip_user_perms()

        response = self.client.get(
            reverse("wagtailsnippets:list", args=['wagtail_localize_test', 'testsnippet'])
        )

        self.assertNotContains(response, "Sync translated snippets")


@override_settings(
    LANGUAGES=[
        ('en', "English"),
        ('fr', "French"),
        ('de', "German"),
        ('es', "Spanish"),
    ],
    WAGTAIL_CONTENT_LANGUAGES=[
        ('en', "English"),
        ('fr', "French"),
        ('de', "German"),
        ('es', "Spanish"),
    ],
)
class TestUpdateTranslations(TestCase, WagtailTestUtils):
    def setUp(self):
        self.login()

        self.en_locale = Locale.objects.get()
        self.fr_locale = Locale.objects.create(language_code="fr")
        self.de_locale = Locale.objects.create(language_code="de")

        self.en_homepage = Page.objects.get(depth=2)
        self.fr_homepage = self.en_homepage.copy_for_translation(self.fr_locale)
        self.de_homepage = self.en_homepage.copy_for_translation(self.de_locale)

        self.en_blog_index = make_test_page(self.en_homepage, title="Blog", slug="blog")
        self.en_blog_post = make_test_page(
            self.en_blog_index, title="Blog post", slug="blog-post", test_charfield="Test content"
        )

        self.en_snippet = TestSnippet.objects.create(field="Test snippet")

        # Create translation of page. This creates the FR version
        self.page_source, created = TranslationSource.get_or_create_from_instance(self.en_blog_post)
        self.page_translation = Translation.objects.create(
            source=self.page_source,
            target_locale=self.fr_locale
        )
        self.page_translation.save_target(publish=True)
        self.fr_blog_post = self.en_blog_post.get_translation(self.fr_locale)

        # Create translation of snippet. This creates the FR version
        self.snippet_source, created = TranslationSource.get_or_create_from_instance(self.en_snippet)
        self.snippet_translation = Translation.objects.create(
            source=self.snippet_source,
            target_locale=self.fr_locale
        )
        self.snippet_translation.save_target(publish=True)
        self.fr_snippet = self.en_snippet.get_translation(self.fr_locale)

    def test_get_update_page_translation(self):
        response = self.client.get(
            reverse(
                "wagtail_localize:update_translations",
                args=[self.page_source.id],
            )
        )

        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            response.context['translations'],
            [
                {
                    'title': 'Blog post',
                    'locale': self.fr_locale,
                    'edit_url': reverse('wagtailadmin_pages:edit', args=[self.fr_blog_post.id])
                }
            ]
        )

    def test_get_update_snippet_translation(self):
        response = self.client.get(
            reverse(
                "wagtail_localize:update_translations",
                args=[self.snippet_source.id],
            )
        )

        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            response.context['translations'],
            [
                {
                    'title': str(self.fr_snippet),
                    'locale': self.fr_locale,
                    'edit_url': reverse('wagtailsnippets:edit', args=['wagtail_localize_test', 'testsnippet', self.fr_snippet.id]),
                }
            ]
        )

    def test_get_without_permission(self):
        strip_user_perms()

        response = self.client.get(
            reverse(
                "wagtail_localize:update_translations",
                args=[self.page_source.id],
            )
        )

        assert_permission_denied(self, response)

    def test_get_with_disabled_translation(self):
        self.page_translation.enabled = False
        self.page_translation.save()

        response = self.client.get(
            reverse(
                "wagtail_localize:update_translations",
                args=[self.page_source.id],
            )
        )

        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            response.context['translations'],
            []
        )

    def test_post_update_page_translation(self):
        self.en_blog_post.test_charfield = "Edited blog post"
        self.en_blog_post.save_revision().publish()

        response = self.client.post(
            reverse(
                "wagtail_localize:update_translations",
                args=[self.page_source.id],
            )
        )

        self.assertRedirects(response, reverse('wagtailadmin_explore', args=[self.en_blog_index.id]))

        # Check that the new string was submitted
        string_segment = StringSegment.objects.get(context__path='test_charfield')
        self.assertEqual(string_segment.string.data, "Edited blog post")

        # The FR version shouldn't be updated yet
        self.fr_blog_post.refresh_from_db()
        self.assertEqual(self.fr_blog_post.test_charfield, "Test content")

    def test_post_update_page_translation_with_publish_translations(self):
        self.en_blog_post.test_charfield = "Edited blog post"
        self.en_blog_post.save_revision().publish()

        response = self.client.post(
            reverse(
                "wagtail_localize:update_translations",
                args=[self.page_source.id],
            ),
            {
                'publish_translations': 'on'
            }
        )

        self.assertRedirects(response, reverse('wagtailadmin_explore', args=[self.en_blog_index.id]))

        # The FR version should be updated
        self.fr_blog_post.refresh_from_db()
        self.assertEqual(self.fr_blog_post.test_charfield, "Edited blog post")

    def test_post_update_snippet_translation(self):
        self.en_snippet.field = "Edited snippet"
        self.en_snippet.save()

        response = self.client.post(
            reverse(
                "wagtail_localize:update_translations",
                args=[self.snippet_source.id],
            )
        )

        self.assertRedirects(response, reverse('wagtailsnippets:edit', args=['wagtail_localize_test', 'testsnippet', self.en_snippet.id]))

        # The FR version shouldn't be updated yet
        self.fr_snippet.refresh_from_db()
        self.assertEqual(self.fr_snippet.field, "Test snippet")

    def test_post_update_snippet_translation_with_publish_translations(self):
        self.en_snippet.field = "Edited snippet"
        self.en_snippet.save()

        response = self.client.post(
            reverse(
                "wagtail_localize:update_translations",
                args=[self.snippet_source.id],
            ),
            {
                'publish_translations': 'on'
            }
        )

        self.assertRedirects(response, reverse('wagtailsnippets:edit', args=['wagtail_localize_test', 'testsnippet', self.en_snippet.id]))

        # The FR version should be updated
        self.fr_snippet.refresh_from_db()
        self.assertEqual(self.fr_snippet.field, "Edited snippet")

    def test_post_with_disabled_translation(self):
        self.page_translation.enabled = False
        self.page_translation.save()

        self.en_blog_post.test_charfield = "Edited blog post"
        self.en_blog_post.save_revision().publish()

        response = self.client.post(
            reverse(
                "wagtail_localize:update_translations",
                args=[self.page_source.id],
            )
        )

        self.assertRedirects(response, reverse('wagtailadmin_explore', args=[self.en_blog_index.id]))

        # Check that the new string was submitted
        string_segment = StringSegment.objects.get(context__path='test_charfield')
        self.assertEqual(string_segment.string.data, "Edited blog post")

        # The FR version shouldn't be updated
        self.fr_blog_post.refresh_from_db()
        self.assertEqual(self.fr_blog_post.test_charfield, "Test content")

    def test_post_with_disabled_translation_with_publish_translations(self):
        self.page_translation.enabled = False
        self.page_translation.save()

        self.en_blog_post.test_charfield = "Edited blog post"
        self.en_blog_post.save_revision().publish()

        response = self.client.post(
            reverse(
                "wagtail_localize:update_translations",
                args=[self.page_source.id],
            ),
            {
                'publish_translations': 'on'
            }
        )

        self.assertRedirects(response, reverse('wagtailadmin_explore', args=[self.en_blog_index.id]))

        # The FR version shouldn't be updated
        self.fr_blog_post.refresh_from_db()
        self.assertEqual(self.fr_blog_post.test_charfield, "Test content")



import uuid
import unittest

from django.test import TestCase

from wagtail.core.blocks import StreamValue
from wagtail.core.models import Page, Locale

from wagtail_localize.fields import copy_synchronised_fields
from wagtail_localize.segments import (
    StringSegmentValue,
    TemplateSegmentValue,
    RelatedObjectSegmentValue,
    OverridableSegmentValue,
)
from wagtail_localize.segments.ingest import ingest_segments
from wagtail_localize.strings import StringValue
from wagtail_localize.test.models import TestPage, TestSnippet, TestChildObject


def make_test_page(**kwargs):
    root_page = Page.objects.get(id=1)
    kwargs.setdefault("title", "Test page")
    return root_page.add_child(instance=TestPage(**kwargs))


RICH_TEXT_TEST_INPUT = '<h1>This is a heading</h1><p>This is a paragraph. &lt;foo&gt; <b>Bold text</b></p><ul><li><a href="http://example.com">This is a link</a></li></ul>'

RICH_TEXT_TEST_FRENCH_SEGMENTS = [
    TemplateSegmentValue(
        "",
        "html",
        '<h1><text position="0"></text></h1><p><text position="1"></text></p><ul><li><text position="2"></text></li></ul>',
        3,
        order=9,
    ),
    StringSegmentValue("", "Ceci est une rubrique", order=10),
    StringSegmentValue.from_source_html(
        "",
        'Ceci est un paragraphe. &lt;foo&gt; <b>Texte en gras</b>',
        order=11,
    ),
    StringSegmentValue(
        "",
        StringValue('<a id="a1">Ceci est un lien</a>'),
        attrs={
            "a1": {"href": "http://example.com"}
        },
        order=12,
    ),
    OverridableSegmentValue(
        "'http://example.com'",
        "http://example.fr",
        order=13,
    )
]

RICH_TEXT_TEST_OUTPUT = '<h1>Ceci est une rubrique</h1><p>Ceci est un paragraphe. &lt;foo&gt; <b>Texte en gras</b></p><ul><li><a href="http://example.fr">Ceci est un lien</a></li></ul>'


class TestSegmentIngestion(TestCase):
    def setUp(self):
        self.src_locale = Locale.get_default()
        self.locale = Locale.objects.create(language_code="fr")

    def test_charfield(self):
        page = make_test_page(test_charfield="Test content")
        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [StringSegmentValue("test_charfield", "Tester le contenu")],
        )

        self.assertEqual(translated_page.test_charfield, "Tester le contenu")

    def test_textfield(self):
        page = make_test_page(test_textfield="Test content")
        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [StringSegmentValue("test_textfield", "Tester le contenu")],
        )

        self.assertEqual(translated_page.test_textfield, "Tester le contenu")

    def test_emailfield(self):
        page = make_test_page(test_emailfield="test@example.com")
        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [StringSegmentValue("test_emailfield", "test@example.fr")],
        )

        self.assertEqual(translated_page.test_emailfield, "test@example.fr")

    def test_slugfield(self):
        page = make_test_page(test_slugfield="test-content")
        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [StringSegmentValue("test_slugfield", "tester-le-contenu")],
        )

        self.assertEqual(translated_page.test_slugfield, "tester-le-contenu")

    def test_urlfield(self):
        page = make_test_page(test_urlfield="http://test-content.com/foo")
        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [StringSegmentValue("test_urlfield", "http://test-content.fr/foo")],
        )

        self.assertEqual(translated_page.test_urlfield, "http://test-content.fr/foo")

    def test_richtextfield(self):
        page = make_test_page(test_richtextfield=RICH_TEXT_TEST_INPUT)
        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [
                segment.wrap("test_richtextfield")
                for segment in RICH_TEXT_TEST_FRENCH_SEGMENTS
            ],
        )

        self.assertEqual(translated_page.test_richtextfield, RICH_TEXT_TEST_OUTPUT)

    def test_snippet(self):
        test_snippet = TestSnippet.objects.create(field="Test content")
        translated_snippet = test_snippet.copy_for_translation(self.locale)
        translated_snippet.save()

        # Ingest segments into the snippet
        ingest_segments(
            test_snippet,
            translated_snippet,
            self.src_locale,
            self.locale,
            [StringSegmentValue("field", "Tester le contenu")],
        )

        translated_snippet.save()

        self.assertEqual(translated_snippet.field, "Tester le contenu")

        # Now ingest a RelatedObjectSegmentValue into the page
        page = make_test_page(test_snippet=test_snippet)
        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [RelatedObjectSegmentValue.from_instance("test_snippet", test_snippet)],
        )

        # Check the translated snippet was linked to the translated page
        self.assertNotEqual(page.test_snippet_id, translated_page.test_snippet_id)
        self.assertEqual(page.test_snippet.locale, self.src_locale)
        self.assertEqual(translated_page.test_snippet.locale, self.locale)
        self.assertEqual(
            page.test_snippet.translation_key,
            translated_page.test_snippet.translation_key,
        )

        self.assertEqual(translated_page.test_snippet.field, "Tester le contenu")

    def test_childobjects(self):
        page = make_test_page()
        page.test_childobjects.add(TestChildObject(field="Test content"))
        page.save()

        child_translation_key = TestChildObject.objects.get().translation_key

        translated_page = page.copy_for_translation(self.locale)

        copy_synchronised_fields(page, translated_page)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [
                StringSegmentValue(
                    f"test_childobjects.{child_translation_key}.field",
                    "Tester le contenu",
                )
            ],
        )

        old_child_object = page.test_childobjects.get()
        new_child_object = translated_page.test_childobjects.get()

        self.assertNotEqual(old_child_object.id, new_child_object.id)
        self.assertEqual(old_child_object.locale, self.src_locale)
        self.assertEqual(new_child_object.locale, self.locale)
        self.assertEqual(
            old_child_object.translation_key, new_child_object.translation_key
        )

        self.assertEqual(new_child_object.field, "Tester le contenu")

    def test_customfield(self):
        page = make_test_page(test_customfield="Test content")

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [StringSegmentValue("test_customfield.foo", "Tester le contenu")],
        )

        self.assertEqual(translated_page.test_customfield, "Tester le contenu")


def make_test_page_with_streamfield_block(block_id, block_type, block_value, **kwargs):
    stream_data = [{"id": block_id, "type": block_type, "value": block_value}]

    return make_test_page(
        test_streamfield=StreamValue(
            TestPage.test_streamfield.field.stream_block, stream_data, is_lazy=True
        ),
        **kwargs,
    )


class TestSegmentIngestionWithStreamField(TestCase):
    def setUp(self):
        self.src_locale = Locale.get_default()
        self.locale = Locale.objects.create(language_code="fr")

    def test_charblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_charblock", "Test content"
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [StringSegmentValue(f"test_streamfield.{block_id}", "Tester le contenu")],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_charblock",
                    "value": "Tester le contenu",
                }
            ],
        )

    def test_textblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_textblock", "Test content"
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [StringSegmentValue(f"test_streamfield.{block_id}", "Tester le contenu")],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_textblock",
                    "value": "Tester le contenu",
                }
            ],
        )

    def test_emailblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_emailblock", "test@example.com"
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [StringSegmentValue(f"test_streamfield.{block_id}", "test@example.fr")],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_emailblock",
                    "value": "test@example.fr",
                }
            ],
        )

    def test_urlblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_urlblock", "http://test-content.com/foo"
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [
                StringSegmentValue(
                    f"test_streamfield.{block_id}", "http://test-content.fr/foo"
                )
            ],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_urlblock",
                    "value": "http://test-content.fr/foo",
                }
            ],
        )

    def test_embedblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_embedblock", "https://www.youtube.com/watch?v=pGpCHRyUZXQ"
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [
                OverridableSegmentValue(
                    f"test_streamfield.{block_id}", "https://www.youtube.com/watch?v=aBByJQCaaEA"
                )
            ],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_embedblock",
                    "value": "https://www.youtube.com/watch?v=aBByJQCaaEA",
                }
            ],
        )

    def test_richtextblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_richtextblock", RICH_TEXT_TEST_INPUT
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [
                segment.wrap(f"test_streamfield.{block_id}")
                for segment in RICH_TEXT_TEST_FRENCH_SEGMENTS
            ],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_richtextblock",
                    "value": RICH_TEXT_TEST_OUTPUT,
                }
            ],
        )

    @unittest.expectedFailure  # Not supported
    def test_rawhtmlblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_rawhtmlblock", RICH_TEXT_TEST_INPUT
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [
                segment.wrap(f"test_streamfield.{block_id}")
                for segment in RICH_TEXT_TEST_FRENCH_SEGMENTS
            ],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_rawhtmlblock",
                    "value": RICH_TEXT_TEST_OUTPUT,
                }
            ],
        )

    def test_blockquoteblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_blockquoteblock", "Test content"
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [StringSegmentValue(f"test_streamfield.{block_id}", "Tester le contenu")],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_blockquoteblock",
                    "value": "Tester le contenu",
                }
            ],
        )

    def test_structblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id),
            "test_structblock",
            {"field_a": "Test content", "field_b": "Some more test content"},
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [
                StringSegmentValue(
                    f"test_streamfield.{block_id}.field_a", "Tester le contenu"
                ),
                StringSegmentValue(
                    f"test_streamfield.{block_id}.field_b", "Encore du contenu de test"
                ),
            ],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_structblock",
                    "value": {
                        "field_a": "Tester le contenu",
                        "field_b": "Encore du contenu de test",
                    },
                }
            ],
        )

    @unittest.expectedFailure  # Not supported (probably won't ever be due to lack of path stability)
    def test_listblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id), "test_listblock", ["Test content", "Some more test content"]
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [
                StringSegmentValue(
                    f"test_streamfield.{block_id}", "Tester le contenu", order=0
                ),
                StringSegmentValue(
                    f"test_streamfield.{block_id}", "Encore du contenu de test", order=1
                ),
            ],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_listblock",
                    "value": ["Tester le contenu", "Encore du contenu de test"],
                }
            ],
        )

    def test_nestedstreamblock(self):
        block_id = uuid.uuid4()
        nested_block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id),
            "test_nestedstreamblock",
            [{"id": str(nested_block_id), "type": "block_a", "value": "Test content"}],
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [
                StringSegmentValue(
                    f"test_streamfield.{block_id}.{nested_block_id}",
                    "Tester le contenu",
                )
            ],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_nestedstreamblock",
                    "value": [
                        {
                            "id": str(nested_block_id),
                            "type": "block_a",
                            "value": "Tester le contenu",
                        }
                    ],
                }
            ],
        )

    def test_customstructblock(self):
        block_id = uuid.uuid4()
        page = make_test_page_with_streamfield_block(
            str(block_id),
            "test_customstructblock",
            {"field_a": "Test content", "field_b": "Some more test content"},
        )

        translated_page = page.copy_for_translation(self.locale)

        ingest_segments(
            page,
            translated_page,
            self.src_locale,
            self.locale,
            [
                StringSegmentValue(
                    f"test_streamfield.{block_id}.foo",
                    "Tester le contenu / Encore du contenu de test",
                )
            ],
        )

        translated_page.save()
        translated_page.refresh_from_db()

        self.assertEqual(
            list(translated_page.test_streamfield.stream_data),
            [
                {
                    "id": str(block_id),
                    "type": "test_customstructblock",
                    "value": {
                        "field_a": "Tester le contenu",
                        "field_b": "Encore du contenu de test",
                    },
                }
            ],
        )



# Generated by Django 3.0.8 on 2020-08-11 08:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize', '0006_create_submit_translation_permission'),
    ]

    operations = [
        migrations.AddField(
            model_name='stringtranslation',
            name='tool_name',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name='stringtranslation',
            name='translation_type',
            field=models.CharField(choices=[('manual', 'Manual'), ('machine', 'Machine')], default='manual', max_length=20),
            preserve_default=False,
        ),
    ]



from unittest import TestCase

from wagtail_localize.segments import StringSegmentValue


class TestStringSegmentValue(TestCase):
    def test_segment_value(self):
        segment = StringSegmentValue.from_source_html(
            "foo.bar",
            'This is some text. &lt;foo&gt; <b>Bold text</b> <a href="http://example.com">A link and some more <b>Bold text</b></a>',
        )

        self.assertEqual(segment.path, "foo.bar")
        self.assertEqual(segment.order, 0)
        self.assertEqual(
            segment.attrs, {"a1": {"href": "http://example.com"}}
        )
        self.assertEqual(
            segment.render_text(),
            "This is some text. <foo> Bold text A link and some more Bold text",
        )
        self.assertEqual(
            segment.render_html(),
            'This is some text. &lt;foo&gt; <b>Bold text</b> <a href="http://example.com">A link and some more <b>Bold text</b></a>',
        )

        # .with_order()
        orderred = segment.with_order(123)
        self.assertEqual(segment.order, 0)
        self.assertEqual(orderred.order, 123)
        self.assertEqual(orderred.path, "foo.bar")
        self.assertEqual(orderred.string, segment.string)
        self.assertEqual(orderred.attrs, segment.attrs)

        # .wrap()
        wrapped = segment.wrap("baz")
        self.assertEqual(segment.path, "foo.bar")
        self.assertEqual(wrapped.path, "baz.foo.bar")
        self.assertEqual(wrapped.order, segment.order)
        self.assertEqual(wrapped.string, segment.string)
        self.assertEqual(wrapped.attrs, segment.attrs)

        # .unwrap()
        path_component, unwrapped = segment.unwrap()
        self.assertEqual(segment.path, "foo.bar")
        self.assertEqual(path_component, "foo")
        self.assertEqual(unwrapped.path, "bar")
        self.assertEqual(unwrapped.order, segment.order)
        self.assertEqual(unwrapped.string, segment.string)
        self.assertEqual(unwrapped.attrs, segment.attrs)

    def test_replace_html_attrs(self):
        segment = StringSegmentValue.from_source_html(
            "foo.bar",
            'This is some text. &lt;foo&gt; <b>Bold text</b> <a id="a1">A link and some more <b>Bold text</b></a>',
        )

        segment.attrs = {"a1": {"href": "http://changed-example.com"}}

        self.assertEqual(
            segment.render_html(),
            'This is some text. &lt;foo&gt; <b>Bold text</b> <a href="http://changed-example.com">A link and some more <b>Bold text</b></a>',
        )

    def test_raises_typeerror_if_string_is_none(self):
        with self.assertRaises(TypeError) as e:
            StringSegmentValue("foo.bar", None)

        self.assertEqual(str(e.exception), '`string` must be either a `StringValue` or a `str`. Got `NoneType`')



# Generated by Django 3.1.3 on 2020-11-10 16:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize_test', '0008_flaglocalecomponent'),
    ]

    operations = [
        migrations.AlterField(
            model_name='testpage',
            name='test_charfield',
            field=models.CharField(blank=True, default='', max_length=255, null=True, verbose_name='char field'),
        ),
        migrations.DeleteModel(
            name='FlagLocaleComponent',
        ),
    ]



# Generated by Django 3.0.8 on 2020-07-29 11:30

from django.db import migrations


def create_submit_translation_permission(apps, schema_editor):
    ContentType = apps.get_model('contenttypes.ContentType')
    Permission = apps.get_model('auth.Permission')
    Group = apps.get_model('auth.Group')

    # Add a fake content type to hang the 'Can submit translations' permission off.
    # The fact that this doesn't correspond to an actual defined model shouldn't matter, I hope...
    wagtailadmin_content_type, created = ContentType.objects.get_or_create(
        app_label='wagtail_localize',
        model='translation'
    )

    # Create admin permission
    submit_permission, created = Permission.objects.get_or_create(
        content_type=wagtailadmin_content_type,
        codename='submit_translation',
        name='Can submit translations'
    )

    # Assign it to Editors and Moderators groups
    for group in Group.objects.filter(name__in=['Editors', 'Moderators']):
        group.permissions.add(submit_permission)


def remove_submit_translation_permission(apps, schema_editor):
    ContentType = apps.get_model('contenttypes.ContentType')
    Permission = apps.get_model('auth.Permission')
    wagtailadmin_content_type = ContentType.objects.get(
        app_label='wagtail_localize',
        model='translation',
    )
    # This cascades to Group
    Permission.objects.filter(
        content_type=wagtailadmin_content_type,
        codename='submit_translation',
    ).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize', '0005_remove_translationsource_object'),
    ]

    operations = [
        migrations.RunPython(create_submit_translation_permission, remove_submit_translation_permission),
    ]



from django.apps import AppConfig


class WagtailLocalizeAppConfig(AppConfig):
    name = 'wagtail_localize'
    label = 'wagtail_localize'
    verbose_name = "Wagtail localize"

    def ready(self):
        from .models import register_post_delete_signal_handlers
        register_post_delete_signal_handlers()



# Generated by Django 3.0.9 on 2020-08-27 16:18

from django.db import migrations
import django.db.models.deletion
import modelcluster.fields


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize_test', '0003_nontranslatablechildobject_testgeneratetranslatablefieldspage_translatablechildobject'),
    ]

    operations = [
        migrations.AlterField(
            model_name='nontranslatablechildobject',
            name='page',
            field=modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='test_nontranslatable_childobjects', to='wagtail_localize_test.TestGenerateTranslatableFieldsPage'),
        ),
        migrations.AlterField(
            model_name='translatablechildobject',
            name='page',
            field=modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='test_translatable_childobjects', to='wagtail_localize_test.TestGenerateTranslatableFieldsPage'),
        ),
    ]



import json

from django.contrib.contenttypes.models import ContentType
from django.test import TestCase
from django.utils import timezone
from wagtail.core.blocks import StreamValue
from wagtail.core.models import Page, Locale, PageLogEntry

from wagtail_localize.models import (
    TranslationSource,
    String,
    StringTranslation,
    SourceDeletedError,
    MissingTranslationError,
    MissingRelatedObjectError,
    TranslationContext,
)
from wagtail_localize.segments import RelatedObjectSegmentValue
from wagtail_localize.strings import StringValue
from wagtail_localize.test.models import TestPage, TestSnippet, TestChildObject, TestSynchronizedChildObject, TestNonParentalChildObject


def create_test_page(**kwargs):
    parent = kwargs.pop("parent", None) or Page.objects.get(id=1)
    page = parent.add_child(instance=TestPage(**kwargs))
    page_revision = page.save_revision()
    page_revision.publish()
    page.refresh_from_db()

    source, created = TranslationSource.get_or_create_from_instance(page)

    prepare_source(source)

    return page


def prepare_source(source):
    # Recurse into any related objects
    for segment in source.relatedobjectsegment_set.all():
        if not isinstance(segment, RelatedObjectSegmentValue):
            continue

        related_source, created = TranslationSource.get_or_create_from_instance(
            segment.get_instance(source.locale)
        )
        prepare_source(related_source)


class TestGetOrCreateFromInstance(TestCase):
    def setUp(self):
        self.snippet = TestSnippet.objects.create(field="This is some test content")

    def test_create(self):
        source, created = TranslationSource.get_or_create_from_instance(self.snippet)

        self.assertTrue(created)
        self.assertEqual(source.object_id, self.snippet.translation_key)
        self.assertEqual(source.locale, self.snippet.locale)
        self.assertEqual(
            json.loads(source.content_json),
            {
                "pk": self.snippet.pk,
                "field": "This is some test content",
                "translation_key": str(self.snippet.translation_key),
                "locale": self.snippet.locale_id,
            },
        )
        self.assertTrue(source.created_at)

    def test_update(self):
        source = TranslationSource.objects.create(
            object_id=self.snippet.translation_key,
            specific_content_type=ContentType.objects.get_for_model(TestSnippet),
            locale=self.snippet.locale,
            content_json=json.dumps(
                {
                    "pk": self.snippet.pk,
                    "field": "Some different content",  # Changed
                    "translation_key": str(self.snippet.translation_key),
                    "locale": self.snippet.locale_id,
                }
            ),
            last_updated_at=timezone.now(),
        )

        new_source, created = TranslationSource.get_or_create_from_instance(self.snippet)

        self.assertEqual(source, new_source)
        self.assertEqual(
            json.loads(source.content_json)["field"], "Some different content"
        )
        self.assertEqual(
            json.loads(new_source.content_json)["field"], "Some different content"
        )


class TestUpdateOrCreateFromInstance(TestCase):
    def setUp(self):
        self.snippet = TestSnippet.objects.create(field="This is some test content")

    def test_create(self):
        source, created = TranslationSource.update_or_create_from_instance(self.snippet)

        self.assertTrue(created)
        self.assertEqual(source.object_id, self.snippet.translation_key)
        self.assertEqual(source.locale, self.snippet.locale)
        self.assertEqual(
            json.loads(source.content_json),
            {
                "pk": self.snippet.pk,
                "field": "This is some test content",
                "translation_key": str(self.snippet.translation_key),
                "locale": self.snippet.locale_id,
            },
        )
        self.assertTrue(source.created_at)

    def test_update(self):
        source = TranslationSource.objects.create(
            object_id=self.snippet.translation_key,
            specific_content_type=ContentType.objects.get_for_model(TestSnippet),
            locale=self.snippet.locale,
            content_json=json.dumps(
                {
                    "pk": self.snippet.pk,
                    "field": "Some different content",  # Changed
                    "translation_key": str(self.snippet.translation_key),
                    "locale": self.snippet.locale_id,
                }
            ),
            last_updated_at=timezone.now(),
        )

        new_source, created = TranslationSource.update_or_create_from_instance(self.snippet)

        self.assertFalse(created)

        self.assertEqual(source, new_source)
        self.assertEqual(
            json.loads(source.content_json)["field"], "Some different content"
        )
        self.assertEqual(
            json.loads(new_source.content_json)["field"], "This is some test content"
        )


class TestAsInstanceForPage(TestCase):
    def setUp(self):
        self.page = create_test_page(title="Test page", slug="test-page")
        self.source, created = TranslationSource.get_or_create_from_instance(self.page)

    def test(self):
        # To show it actually is using the translation source and not the live object,
        # mess with the JSON content manually
        content = json.loads(self.source.content_json)

        content["title"] = "Changed title"
        self.source.content_json = json.dumps(content)
        self.source.save(update_fields=["content_json"])

        new_instance = self.source.as_instance()

        self.assertIsInstance(new_instance, TestPage)
        self.assertEqual(new_instance.id, self.page.id)
        self.assertEqual(new_instance.title, "Changed title")

    def test_raises_error_if_source_deleted(self):
        self.page.delete()

        with self.assertRaises(SourceDeletedError):
            self.source.as_instance()


class TestAsInstanceForSnippet(TestCase):
    def setUp(self):
        self.snippet = TestSnippet.objects.create(field="This is some test content")
        self.source, created = TranslationSource.get_or_create_from_instance(self.snippet)

    def test(self):
        # To show it actually is using the translation source and not the live object,
        # mess with the JSON content manually
        content = json.loads(self.source.content_json)

        content["field"] = "Some changed content"
        self.source.content_json = json.dumps(content)
        self.source.save(update_fields=["content_json"])

        new_instance = self.source.as_instance()

        self.assertIsInstance(new_instance, TestSnippet)
        self.assertEqual(new_instance.id, self.snippet.id)
        self.assertEqual(new_instance.field, "Some changed content")


class TestExportPO(TestCase):
    def setUp(self):
        self.page = create_test_page(title="Test page", slug="test-page", test_charfield="This is some test content", test_textfield="This is some test content")
        self.source, created = TranslationSource.get_or_create_from_instance(self.page)

    def test_export_po(self):
        po = self.source.export_po()

        self.assertEqual(po.metadata.keys(), {'POT-Creation-Date', 'MIME-Version', 'Content-Type'})
        self.assertEqual(po.metadata['MIME-Version'], '1.0')
        self.assertEqual(po.metadata['Content-Type'], 'text/plain; charset=utf-8')

        self.assertEqual(len(po), 2)
        self.assertEqual(po[0].msgid, "This is some test content")
        self.assertEqual(po[0].msgctxt, "test_charfield")
        self.assertEqual(po[0].msgstr, "")
        self.assertFalse(po[0].obsolete)

        # Test with the exact same content twice
        # This checks for https://github.com/wagtail/wagtail-localize/issues/253
        self.assertEqual(po[1].msgid, "This is some test content")
        self.assertEqual(po[1].msgctxt, "test_textfield")
        self.assertEqual(po[1].msgstr, "")
        self.assertFalse(po[1].obsolete)


class TestCreateOrUpdateTranslationForPage(TestCase):
    def setUp(self):
        self.snippet = TestSnippet.objects.create(field="Test snippet content")
        self.page = create_test_page(
            title="Test page",
            slug="test-page",
            test_charfield="This is some test content",
            test_snippet=self.snippet,
        )
        self.source, created = TranslationSource.get_or_create_from_instance(self.page)
        self.source_locale = Locale.objects.get(language_code="en")
        self.dest_locale = Locale.objects.create(language_code="fr")

        # Translate the snippet
        self.translated_snippet = self.snippet.copy_for_translation(self.dest_locale)
        self.translated_snippet.field = "Tester le contenu de l'extrait"
        self.translated_snippet.save()

        # Add translation for test_charfield
        self.string = String.from_value(
            self.source_locale, StringValue.from_plaintext("This is some test content")
        )
        self.translation = StringTranslation.objects.create(
            translation_of=self.string,
            locale=self.dest_locale,
            context=TranslationContext.objects.get(
                object_id=self.page.translation_key, path="test_charfield"
            ),
            data="Ceci est du contenu de test",
        )

    def test_create(self):
        new_page, created = self.source.create_or_update_translation(self.dest_locale)

        self.assertTrue(created)
        self.assertEqual(new_page.title, "Test page")
        self.assertEqual(new_page.slug, 'test-page-fr')
        self.assertEqual(new_page.test_charfield, "Ceci est du contenu de test")
        self.assertEqual(new_page.translation_key, self.page.translation_key)
        self.assertEqual(new_page.locale, self.dest_locale)
        self.assertTrue(
            self.source.translation_logs.filter(locale=self.dest_locale).exists()
        )

    def test_create_finds_available_slug(self):
        # Create a page with the `test-page-fr` slug already
        create_test_page(
            title="Test Page FR",
            slug="test-page-fr",
        )

        new_page, created = self.source.create_or_update_translation(self.dest_locale)

        self.assertTrue(created)
        self.assertEqual(new_page.slug, 'test-page-fr-1')

    def test_create_child(self):
        child_page = create_test_page(
            title="Child page",
            slug="child-page",
            parent=self.page,
            test_charfield="This is some test content",
            test_snippet=self.snippet,
        )
        child_source, created = TranslationSource.get_or_create_from_instance(child_page)

        translated_parent = self.page.copy_for_translation(self.dest_locale)

        # Create a translation for the new context
        StringTranslation.objects.create(
            translation_of=self.string,
            locale=self.dest_locale,
            context=TranslationContext.objects.get(
                object_id=child_page.translation_key, path="test_charfield"
            ),
            data="Ceci est du contenu de test",
        )

        new_page, created = child_source.create_or_update_translation(
            self.dest_locale
        )

        self.assertTrue(created)
        self.assertEqual(new_page.get_parent(), translated_parent)
        self.assertEqual(new_page.title, "Child page")
        self.assertEqual(new_page.test_charfield, "Ceci est du contenu de test")
        self.assertEqual(new_page.test_snippet, self.translated_snippet)
        self.assertEqual(new_page.translation_key, child_page.translation_key)
        self.assertEqual(new_page.locale, self.dest_locale)
        self.assertTrue(
            child_source.translation_logs.filter(locale=self.dest_locale).exists()
        )

    def test_update(self):
        self.page.copy_for_translation(self.dest_locale)

        new_page, created = self.source.create_or_update_translation(self.dest_locale)

        self.assertFalse(created)
        self.assertEqual(new_page.title, "Test page")
        self.assertEqual(new_page.test_charfield, "Ceci est du contenu de test")
        self.assertEqual(new_page.test_snippet, self.translated_snippet)
        self.assertEqual(new_page.translation_key, self.page.translation_key)
        self.assertEqual(new_page.locale, self.dest_locale)
        self.assertTrue(
            self.source.translation_logs.filter(locale=self.dest_locale).exists()
        )

    def test_update_synchronised_fields(self):
        # Add a couple of initial child objects.
        # The first one will be deleted in the source page after initial translation. The sync should carry this deletion across.
        # The second won't be deleted so should remain in the translation
        self.page.test_synchronized_childobjects.add(
            TestSynchronizedChildObject(
                field="Test child object that existed before initial translation and was deleted"
            )
        )

        self.page.test_synchronized_childobjects.add(
            TestSynchronizedChildObject(
                field="Test child object that existed before initial translation and was not deleted"
            )
        )

        translated = self.page.copy_for_translation(self.dest_locale)

        self.page.test_synchronized_charfield = "Test synchronised content"
        self.page.test_synchronized_textfield = "Test synchronised content"
        self.page.test_synchronized_emailfield = "test@synchronised.content"
        self.page.test_synchronized_slugfield = "test-synchronised-content"
        self.page.test_synchronized_urlfield = "https://test.synchronised/content"
        self.page.test_synchronized_richtextfield = "<p>Test synchronised content</p>"
        # self.page.test_synchronized_streamfield = ""
        synchronized_snippet = TestSnippet.objects.create(field="Synchronised snippet")
        self.page.test_synchronized_snippet = synchronized_snippet
        self.page.test_synchronized_customfield = "Test synchronised content"

        # Add some child objects
        self.page.test_childobjects.add(
            TestChildObject(
                field="A test child object"
            )
        )

        # Remove one of the child objects to test that deletions are syncrhonised
        self.page.test_synchronized_childobjects.remove(self.page.test_synchronized_childobjects.get(field="Test child object that existed before initial translation and was deleted"))
        self.page.test_synchronized_childobjects.add(
            TestSynchronizedChildObject(
                field="A test synchronized object"
            )
        )

        # Non-parental child objects are created differently because modelcluster doesn't help us
        TestNonParentalChildObject.objects.create(
            page=self.page,
            field="A non-parental child object"
        )

        # Save the page
        revision = self.page.save_revision()
        revision.publish()
        self.page.refresh_from_db()
        source_with_changed_content, created = TranslationSource.update_or_create_from_instance(self.page)

        # Check translation hasn't been updated yet
        translated.refresh_from_db()
        self.assertEqual(translated.test_synchronized_charfield, "")

        # Update the original page again. This will make sure it's taking the content from the translation source and not the live version
        self.page.test_synchronized_charfield = (
            "Test synchronised content updated again"
        )
        self.page.save_revision().publish()

        (
            new_page,
            created,
        ) = source_with_changed_content.create_or_update_translation(
            self.dest_locale,
            fallback=True,
        )

        self.assertFalse(created)
        self.assertEqual(new_page, translated)
        self.assertEqual(
            new_page.test_synchronized_charfield, "Test synchronised content"
        )
        self.assertEqual(
            new_page.test_synchronized_charfield, "Test synchronised content"
        )
        self.assertEqual(
            new_page.test_synchronized_textfield, "Test synchronised content"
        )
        self.assertEqual(
            new_page.test_synchronized_emailfield, "test@synchronised.content"
        )
        self.assertEqual(
            new_page.test_synchronized_slugfield, "test-synchronised-content"
        )
        self.assertEqual(
            new_page.test_synchronized_urlfield, "https://test.synchronised/content"
        )
        self.assertEqual(
            new_page.test_synchronized_richtextfield, "<p>Test synchronised content</p>"
        )
        self.assertEqual(new_page.test_synchronized_snippet, synchronized_snippet)
        self.assertEqual(
            new_page.test_synchronized_customfield, "Test synchronised content"
        )

        # Translatable child objects should always be synchronised
        # Locale should be updated but translation key should be the same
        translatable_child_object = new_page.test_childobjects.get()
        self.assertEqual(translatable_child_object.field, "A test child object")
        self.assertEqual(translatable_child_object.locale, self.dest_locale)
        self.assertTrue(translatable_child_object.has_translation(self.source_locale))

        # Test both the non deleted and new synchronised child objects remain
        self.assertEqual(
            set(new_page.test_synchronized_childobjects.values_list('field', flat=True)),
            {"A test synchronized object", "Test child object that existed before initial translation and was not deleted"}
        )

        # Non parental child objects should be ignored
        self.assertFalse(new_page.test_nonparentalchildobjects.exists())

    def test_update_streamfields(self):
        # Streamfields are special in that they contain content that needs to be synchronised as well as
        # translatable content.

        # Copy page for translation, this will have a blank streamfield
        translated = self.page.copy_for_translation(self.dest_locale)

        # Set streamfield value on original
        self.page.test_streamfield = StreamValue(
            TestPage.test_streamfield.field.stream_block,
            [
                {
                    "id": "id",
                    "type": "test_charblock",
                    "value": "This is some test content",
                }
            ],
            is_lazy=True,
        )

        # Save the page
        revision = self.page.save_revision()
        revision.publish()
        self.page.refresh_from_db()
        source_with_streamfield, created = TranslationSource.update_or_create_from_instance(self.page)

        # Create a translation for the new context
        StringTranslation.objects.create(
            translation_of=self.string,
            locale=self.dest_locale,
            context=TranslationContext.objects.get(
                object_id=self.page.translation_key, path="test_streamfield.id"
            ),
            data="Ceci est du contenu de test",
        )

        new_page, created = source_with_streamfield.create_or_update_translation(
            self.dest_locale
        )

        self.assertFalse(created)
        self.assertEqual(new_page, translated)

        # Check the block was copied into translation
        self.assertEqual(new_page.test_streamfield[0].id, "id")
        self.assertEqual(
            new_page.test_streamfield[0].value, "Ceci est du contenu de test"
        )

    def test_create_translations_not_ready(self):
        self.translation.delete()

        with self.assertRaises(MissingTranslationError) as e:
            self.source.create_or_update_translation(self.dest_locale)

        self.assertEqual(e.exception.segment.source, self.source)
        self.assertEqual(e.exception.segment.context.path, "test_charfield")
        self.assertEqual(e.exception.segment.string, self.string)
        self.assertEqual(e.exception.locale, self.dest_locale)

    def test_create_related_object_not_ready(self):
        self.translated_snippet.delete()

        with self.assertRaises(MissingRelatedObjectError) as e:
            self.source.create_or_update_translation(self.dest_locale)

        self.assertEqual(e.exception.segment.source, self.source)
        self.assertEqual(e.exception.segment.context.path, "test_snippet")
        self.assertEqual(e.exception.segment.object_id, self.snippet.translation_key)
        self.assertEqual(e.exception.locale, self.dest_locale)

    def test_create_not_ready_with_fallback_true(self):
        # Same as previous two tests, except we pass `fallback=True` this time
        # so it falls back to the source value instead of raising an exception.
        self.translation.delete()
        self.translated_snippet.delete()

        translated_page, created = self.source.create_or_update_translation(self.dest_locale, fallback=True)

        self.assertEqual(translated_page.test_snippet, self.snippet)
        self.assertEqual(translated_page.test_charfield, "This is some test content")

    def test_create_with_fallback_true(self):
        # Like the previous test, but this time we have valid data, so it shouldn't fallback
        translated_page, created = self.source.create_or_update_translation(self.dest_locale, fallback=True)

        self.assertEqual(translated_page.test_snippet, self.translated_snippet)
        self.assertEqual(translated_page.test_charfield, "Ceci est du contenu de test")

    def test_convert_alias(self):
        self.page.copy_for_translation(self.dest_locale, alias=True)

        new_page, created = self.source.create_or_update_translation(self.dest_locale)

        self.assertFalse(created)
        self.assertIsNone(new_page.alias_of)
        self.assertEqual(new_page.title, "Test page")
        self.assertEqual(new_page.slug, 'test-page-fr')
        self.assertEqual(new_page.test_charfield, "Ceci est du contenu de test")
        self.assertEqual(new_page.translation_key, self.page.translation_key)
        self.assertEqual(new_page.locale, self.dest_locale)
        self.assertTrue(
            self.source.translation_logs.filter(locale=self.dest_locale).exists()
        )

        # Check a log was created for the alias conversion
        self.assertTrue(PageLogEntry.objects.filter(page=new_page, action='wagtail.convert_alias').exists())


class TestGetEphemeralTranslatedInstance(TestCase):
    def setUp(self):
        self.page = create_test_page(
            title="Test page",
            slug="test-page",
            test_charfield="This is some test content",
            test_textfield="This is some more test content",
        )
        self.source, created = TranslationSource.get_or_create_from_instance(self.page)
        self.source_locale = Locale.objects.get(language_code="en")
        self.dest_locale = Locale.objects.create(language_code="fr")

        # Translate the page
        self.translated_page = self.page.copy_for_translation(self.dest_locale)

        # Add a test child object and update the test_textfield. Then update the translation source
        self.page.test_childobjects.add(TestChildObject(field="This is a test child object"))
        self.page.test_textfield = "Updated textfield"
        self.page.save_revision().publish()
        self.source.update_from_db()

        # Add translation for test_charfield
        self.translation = StringTranslation.objects.create(
            translation_of=String.objects.get(data="This is some test content"),
            locale=self.dest_locale,
            context=TranslationContext.objects.get(
                object_id=self.page.translation_key, path="test_charfield"
            ),
            data="Ceci est du contenu de test",
        )

        # Add translation for child object
        self.translation = StringTranslation.objects.create(
            translation_of=String.objects.get(data="This is a test child object"),
            locale=self.dest_locale,
            context=TranslationContext.objects.get(
                object_id=self.page.translation_key, path="test_childobjects.{}.field".format(self.page.test_childobjects.get().translation_key)
            ),
            data="Ceci est un objet enfant de test",
        )

    def test_get_ephemeral_translated_instance(self):
        new_page = self.source.get_ephemeral_translated_instance(self.dest_locale, fallback=True)

        self.assertEqual(new_page.id, self.translated_page.id)
        self.assertEqual(new_page.test_charfield, "Ceci est du contenu de test")
        self.assertEqual(new_page.test_textfield, "Updated textfield")
        self.assertEqual(new_page.test_childobjects.get().field, "Ceci est un objet enfant de test")
        self.assertFalse(
            self.source.translation_logs.filter(locale=self.dest_locale).exists()
        )

        # Check the saved page has not been changed
        self.translated_page.refresh_from_db()
        self.assertEqual(self.translated_page.title, "Test page")
        self.assertEqual(self.translated_page.test_charfield, "This is some test content")
        self.assertEqual(self.translated_page.test_textfield, "This is some more test content")
        self.assertFalse(self.translated_page.test_childobjects.exists())

    def test_without_fallback(self):
        # Should raise an error because we haven't translated test_textfield
        with self.assertRaises(MissingTranslationError):
            self.source.get_ephemeral_translated_instance(self.dest_locale)

        # Add a translation for test_textfield to make the error go away
        self.translation = StringTranslation.objects.create(
            translation_of=String.objects.get(data="Updated textfield"),
            locale=self.dest_locale,
            context=TranslationContext.objects.get(
                object_id=self.page.translation_key, path="test_textfield"
            ),
            data="Champ de texte mis à jour",
        )

        new_page = self.source.get_ephemeral_translated_instance(self.dest_locale)
        self.assertEqual(new_page.test_textfield, "Champ de texte mis à jour")


class TestSchemaOutOfDate(TestCase):
    def setUp(self):
        self.page = create_test_page(
            title="Test page",
            slug="test-page",
            test_charfield="This is some test content",
            test_textfield="This is some more test content",
        )
        self.source, created = TranslationSource.get_or_create_from_instance(self.page)

    def test_schema_is_in_date_for_new_instance(self):
        self.assertTrue(self.source.schema_version)
        self.assertFalse(self.source.schema_out_of_date())

    def test_schema_is_in_date_if_source_schema_version_is_unknown(self):
        self.source.schema_version = ''
        self.source.save()
        self.assertFalse(self.source.schema_out_of_date())

    def test_schema_is_out_of_date_if_source_schema_version_is_old(self):
        self.source.schema_version = '0001_initial'
        self.source.save()
        self.assertTrue(self.source.schema_out_of_date())



# Generated by Django 3.0.8 on 2020-08-07 10:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize_test', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='NonTranslatableSnippet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field', models.TextField()),
            ],
        ),
    ]



import requests

from wagtail_localize.strings import StringValue

from .base import BaseMachineTranslator


def language_code(code, is_target=False):
    # DeepL supports targeting Brazillian Portuguese but doesn't have this for other languages
    if is_target and code in ['pt-pt', 'pt-br']:
        return code

    return code.split("-")[0].upper()


class DeepLTranslator(BaseMachineTranslator):
    display_name = "DeepL"

    def translate(self, source_locale, target_locale, strings):
        response = requests.post('https://api.deepl.com/v2/translate', {
            'auth_key': self.options['AUTH_KEY'],
            'text': [string.data for string in strings],
            'tag_handling': 'xml',
            'source_lang': language_code(source_locale.language_code),
            'target_lang': language_code(target_locale.language_code, is_target=True),
        })

        return {
            string: StringValue(translation['text'])
            for string, translation in zip(strings, response.json()['translations'])
        }

    def can_translate(self, source_locale, target_locale):
        return language_code(source_locale.language_code) != language_code(target_locale.language_code, is_target=True)



# Generated by Django 3.1.3 on 2020-11-04 20:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0059_apply_collection_ordering'),
        ('wagtail_localize', '0011_segmentoverride'),
    ]

    operations = [
        migrations.CreateModel(
            name='LocaleSynchronization',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('locale', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.locale')),
                ('sync_from', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.locale')),
            ],
        ),
    ]



LOCALE_COMPONENTS = []


def get_locale_components():
    return LOCALE_COMPONENTS


def register_locale_component(*, heading, help_text=None, required=False):
    def _wrapper(model):
        if model not in LOCALE_COMPONENTS:
            LOCALE_COMPONENTS.append({
                'heading': heading,
                'help_text': help_text,
                'required': required,
                'model': model,
                'slug': model._meta.db_table,
            })
            LOCALE_COMPONENTS.sort(key=lambda x: x['model']._meta.verbose_name)

        return model

    return _wrapper



# Generated by Django 3.1.6 on 2021-02-19 11:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize', '0012_localesynchronization'),
    ]

    operations = [
        migrations.AddField(
            model_name='translationsource',
            name='schema_version',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]



# Generated by Django 3.0.8 on 2020-08-05 09:33

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_localize', '0003_delete_translation_sources'),
    ]

    operations = [
        migrations.AddField(
            model_name='translationsource',
            name='last_updated_at',
            field=models.DateTimeField(default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='translationsource',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterUniqueTogether(
            name='translationsource',
            unique_together={('object', 'locale')},
        ),
    ]



from django.test import TestCase
from wagtail.core.models import Locale

from wagtail_localize.machine_translators.dummy import DummyTranslator
from wagtail_localize.strings import StringValue


class TestDummyTranslator(TestCase):
    def setUp(self):
        self.english_locale = Locale.objects.get()
        self.french_locale = Locale.objects.create(language_code="fr")

    def test_translate_text(self):
        translations = DummyTranslator({}).translate(self.english_locale, self.french_locale, [
            StringValue("Hello world!"),
            StringValue("This is a sentence. This is another sentence."),
        ])

        self.assertEqual(translations, {
            StringValue("Hello world!"): StringValue("world! Hello"),
            StringValue("This is a sentence. This is another sentence."): StringValue("sentence. another is This sentence. a is This"),
        })

    def test_translate_html(self):
        string, attrs = StringValue.from_source_html('<a href="https://en.wikipedia.org/wiki/World">Hello world!</a>. <b>This is a test</b>.')

        translations = DummyTranslator({}).translate(self.english_locale, self.french_locale, [
            string
        ])

        self.assertEqual(translations[string].render_html(attrs), '.<b>test a is This</b>. <a href="https://en.wikipedia.org/wiki/World">world! Hello</a>')

    def test_can_translate(self):
        canada_french_locale = Locale.objects.create(language_code="fr-CA")

        self.assertTrue(DummyTranslator({}).can_translate(self.english_locale, self.french_locale))
        self.assertTrue(DummyTranslator({}).can_translate(self.english_locale, canada_french_locale))

        # Can't translate the same language
        self.assertFalse(DummyTranslator({}).can_translate(self.english_locale, self.english_locale))

        # Can't translate two variants of the same language
        self.assertFalse(DummyTranslator({}).can_translate(self.french_locale, canada_french_locale))



# Generated by Django 3.0.6 on 2020-07-20 17:27

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0057_page_locale_fields_notnull'),
        ('wagtail_localize', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Translation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('source_last_updated_at', models.DateTimeField(auto_now_add=True)),
                ('translations_last_updated_at', models.DateTimeField(null=True)),
                ('destination_last_updated_at', models.DateTimeField(null=True)),
                ('enabled', models.BooleanField(default=True)),
                ('object', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='wagtail_localize.TranslatableObject')),
                ('source', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='wagtail_localize.TranslationSource')),
                ('target_locale', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='translations', to='wagtailcore.Locale')),
            ],
            options={
                'unique_together': {('object', 'target_locale')},
            },
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-30 20:41

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0059_apply_collection_ordering'),
        ('wagtail_localize_test', '0010_testoverridetranslatablefieldspage'),
    ]

    operations = [
        migrations.CreateModel(
            name='TestModelWithInvalidForeignKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('translation_key', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('fk', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.site')),
                ('locale', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='wagtailcore.locale')),
            ],
            options={
                'abstract': False,
                'unique_together': {('translation_key', 'locale')},
            },
        ),
    ]
